/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a source to demonstrate adding various annotations to PDF document.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"
#include "pdfannot.h"

#if defined(FSDK_IOS_DEMO)
#define main pdfannot
#endif


/*******************************************************************************/
/*  class CPDFAnnot												   */
/*******************************************************************************/
CPDFAnnot::CPDFAnnot()
	:m_inputSrcFile("")
	,m_outputFolder("")
	,m_PageRange("")
	,m_inputFolder("")
{

};
CPDFAnnot::~CPDFAnnot()
{

};

/*******************************************************************************/
/*  Set annotation properties.												   */
/*******************************************************************************/

//Set annotation border properties.
void CPDFAnnot::SetAnnotBorder(FSCRT_ANNOT annot)
{
	//Set the border of annotation.
	//Prepare FSPDF_ANNOTBORDER data.
	FSPDF_ANNOTBORDER border;
	border.borderWidth = 2.0;
	border.borderStyle = FSPDF_ANNOT_BORDERSTYLE_DASHED;
	border.cloudIntensity = -1;
	border.dashPhase = 5;
	border.dashCount = 2;
	border.dashPattern[0] = border.dashPattern[1] = 1.0;
	//Set the border.
	FSPDF_Annot_SetBorder(annot, &border);
}

//Set Stamp annotation properties.
void CPDFAnnot::SetStampAnnotProperties(FSCRT_ANNOT annot, const char* sAnnotType, string inputFile)
{
	//Deal with the Highlight annotation only.
	if(strcmp(sAnnotType, FSPDF_ANNOTTYPE_STAMP)) return;

	//Add image as a stamp.
	FSCRT_IMAGE image = NULL;

	CAutoDocument autoDoc((inputFile + "/" + "FoxitLogo.jpg").c_str(),"rb");
	if (!autoDoc.GetFileHander())
	{
		return;
	}

	FS_RESULT ret = FSCRT_Image_LoadFromFile(autoDoc.GetFileHander(),&image);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: In function FSCRT_Image_LoadFromFile with error code %d", ret);
		return;
	}

	ret = FSPDF_Annot_SetStamp(annot,image,0);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: In function FSPDF_Annot_SetStamp with error code %d", ret);
		FSCRT_Image_Release(image);
		return;
	}

	FSDK_OutputLog("Success: Success to set property of stamp annotation.\r\n");
	//release 
	FSCRT_Image_Release(image);
}
//Set Highlight annotation properties.
void CPDFAnnot::SetHighlightAnnotProperties(FSCRT_ANNOT annot, const char* sAnnotType)
{
	//Deal with the Highlight annotation only.
	if(strcmp(sAnnotType, FSPDF_ANNOTTYPE_HIGHLIGHT)) return;

	//Set the quadrilaterals points of annotation.
	FSCRT_QUADPOINTSF quadPoints = {0,0,100,0,0,50,100,50};
	FSPDF_Annot_SetQuadPoints(annot, &quadPoints, 1);
	//Set the stroke color and opacity of annotation.
	FSPDF_Annot_SetColor(annot, FALSE, 0x0000FF00);
	FSPDF_Annot_SetOpacity(annot, (FS_FLOAT)0.55);

	//Add the specified state annotation to an annotation.
	FSCRT_ANNOT stateAnnot = NULL;
	FS_RESULT ret = FSPDF_Annot_AddState(annot, FSPDF_ANNOTSTATEMODEL_REVIEW, FSPDF_ANNOTSTATE_REVIEW_ACCEPTED, &stateAnnot);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		//Remove the state annotation.
		FSPDF_Annot_Remove(stateAnnot);
}

//Set UnderLine annotation properties.
void CPDFAnnot::SetUnderLineAnnotProperties(FSCRT_ANNOT annot, const char* sAnnotType)
{
	//Deal with the UnderLine annotation only.
	if(strcmp(sAnnotType, FSPDF_ANNOTTYPE_UNDERLINE)) return;

	//Set the quadrilaterals points of annotation.
	FSCRT_QUADPOINTSF quadPoints = {0,30,100,30,0,80,100,80};
	FSPDF_Annot_SetQuadPoints(annot, &quadPoints, 1);
	//Set the stroke color and opacity of annotation.
	FSPDF_Annot_SetColor(annot, FALSE, 0x0000FF00);
	FSPDF_Annot_SetOpacity(annot, 1.0);
}

//Set Squiggly annotation properties.
void CPDFAnnot::SetSquigglyAnnotProperties(FSCRT_ANNOT annot, const char* sAnnotType)
{
	//Deal with the Squiggly annotation only.
	if(strcmp(sAnnotType, FSPDF_ANNOTTYPE_SQUIGGLY)) return;

	//Set the quadrilaterals points of annotation.
	FSCRT_QUADPOINTSF quadPoints = {0,50,100,50,0,45,100,45};
	FSPDF_Annot_SetQuadPoints(annot, &quadPoints, 1);
	//Set the stroke color of annotation.
	FSPDF_Annot_SetColor(annot, FALSE, 0x0000FF00);
}

//Set StrikeOut annotation properties.
void CPDFAnnot::SetStrikeOutAnnotProperties(FSCRT_ANNOT annot, const char* sAnnotType)
{
	//Deal with the StrikeOut annotation only.
	if(strcmp(sAnnotType, FSPDF_ANNOTTYPE_STRIKEOUT)) return;

	//Set the quadrilaterals points of annotation.
	FSCRT_QUADPOINTSF quadPoints = {0,30,100,30,0,80,100,80};
	FSPDF_Annot_SetQuadPoints(annot, &quadPoints, 1);
	//Set the stroke color of annotation.
	FSPDF_Annot_SetColor(annot, FALSE, 0x0000FF00);
}

//Set Line annotation properties.
void CPDFAnnot::SetLineAnnotProperties(FSCRT_ANNOT annot, const char* sAnnotType)
{
	//Deal with the Line annotation only.
	if(strcmp(sAnnotType, FSPDF_ANNOTTYPE_LINE)) return;

	//Set the annotation border properties.
	SetAnnotBorder(annot);
	//Set the line ending styles of annotation.
	FSCRT_BSTRC(bsLineStartStyle, FSPDF_ANNOT_LINEENDINGSTYLE_CIRCLE);
	FSCRT_BSTRC(bsLineEndStyle, FSPDF_ANNOT_LINEENDINGSTYLE_BUTT);
	FSPDF_Annot_SetLineEndingStyles(annot, &bsLineStartStyle, &bsLineEndStyle);
	//Set the line position of annotation.
	FSPDF_Annot_SetLinePosition(annot, 0.0, 0.0, 50.0, 50.0);
	//Set the leader line length, extension, offset of annotation.
	FSPDF_Annot_SetLeaderLineLength(annot, 1.0);
	FSPDF_Annot_SetLeaderLineExtension(annot, 10.0);
	FSPDF_Annot_SetLeaderLineOffset(annot, 5.0);
	//Set the flag which decides whether to set a caption to the annotation or not.
	FSPDF_Annot_SetCaptionContents(annot, TRUE);
	FSCRT_BSTRC(bsCaptionPos, "Inline");
	//Set the caption position, offset of annotation.
	FSPDF_Annot_SetCaptionPosition(annot, &bsCaptionPos);
	FSPDF_Annot_SetCaptionOffset(annot, 0.0, 2.5);
}

//Set Polygon annotation properties.
void CPDFAnnot::SetPolygonAnnotProperties(FSCRT_ANNOT annot, const char* sAnnotType)
{
	//Deal with the Polygon annotation only.
	if(strcmp(sAnnotType, FSPDF_ANNOTTYPE_POLYGON)) return;

	//Set the vertices of annotation.
	FS_FLOAT vertices[6] = {300.0, 200.0, 320.0, 130.0, 350.0, 200.0};
	FSPDF_Annot_SetVertices(annot, vertices, 6);
	// Set the fill color and stroke color of the annotation.
	FSPDF_Annot_SetColor(annot, FALSE, 0x00CC00FF);
	FSPDF_Annot_SetColor(annot, TRUE, 0x00FFFF00);
}

//Set Polyline annotation properties.
void CPDFAnnot::SetPolylineAnnotProperties(FSCRT_ANNOT annot, const char* sAnnotType)
{
	//Deal with the Polyline annotation only.
	if(strcmp(sAnnotType, FSPDF_ANNOTTYPE_POLYLINE)) return;

	//Set the vertices of annotation.
	FS_FLOAT vertices[8] = {400.0, 200.0, 420.0, 150.0, 500.0, 200.0, 550.0, 250.0};
	FSPDF_Annot_SetVertices(annot, vertices, 8);
}

//Set Ink annotation properties.
void CPDFAnnot::SetInkAnnotProperties(FSCRT_ANNOT annot, const char* sAnnotType)
{
	//Deal with the Ink annotation only.
	if(strcmp(sAnnotType, FSPDF_ANNOTTYPE_INK)) return;

	//Set the ink list data of annotation.
	//Prepare the ink list data using FSCRT_PATHDATA object.
	FSCRT_PATHDATA pathData;
	//Create a FSCRT_PATHDATA object.
	FS_RESULT ret = FSCRT_PathData_Create(&pathData);
	//Construct the path data.
	FSCRT_PathData_AddPointsCount(pathData, 3);
	FSCRT_PathData_SetPoint(pathData,0,400,300, FSCRT_POINTTYPE_MOVETO);
	FSCRT_PathData_SetPoint(pathData,1,450,330, FSCRT_POINTTYPE_LINETO);
	FSCRT_PathData_SetPoint(pathData,2,500,380, FSCRT_POINTTYPE_LINETO | FSCRT_POINTTYPE_CLOSEFIGURE);
	//Set the Ink List.
	FSPDF_Annot_SetInkList(annot, pathData);
	//Clear the FSCRT_PATHDATA object.
	FSCRT_PathData_Release(pathData);
}

//Set FreeText annotation properties.
void CPDFAnnot::SetFreeTextAnnotProperties(FSCRT_ANNOT annot, const char* sAnnotType)
{
	//Deal with the FreeText annotation only.
	if(strcmp(sAnnotType, FSPDF_ANNOTTYPE_FREETEXT)) return;

	//Set the alignment of annotation.
	FSPDF_Annot_SetAlignment(annot, 0);

	//Set the default appearance of annotation, which can be used in formatting the text.
	//Prepare the default appearance data.
	FSPDF_DEFAULTAPPEARANCE defAppearance;
	defAppearance.flags = FSPDF_DEFAULTAPPEARANCE_FONT | FSPDF_DEFAULTAPPEARANCE_TEXTCOLOR | FSPDF_DEFAULTAPPEARANCE_TEXTMATRIX;
	defAppearance.fontSize = 10.0;
	defAppearance.textColor = 0x0000FF00;
	FSCRT_MATRIX textMatrix = {2, 0, 0, 2, 0, 0};
	defAppearance.textMatrix = textMatrix;
	FSCRT_BSTRC(fontName, "Helvetica");
	FSCRT_Font_Create(&fontName, FSCRT_FONTSTYLE_ITALIC, 400, FSCRT_CHARSET_GB2312, &defAppearance.font);

	//Set the default appearance of annotation.
	FSPDF_Annot_SetDefaultAppearance(annot, &defAppearance);

	FSCRT_Font_Release(defAppearance.font);
}

//Set Link annotation properties.
void CPDFAnnot::SetLinkAnnotProperties(FSCRT_ANNOT annot, const char* sAnnotType)
{
	//Deal with the Link annotation only.
	if(strcmp(sAnnotType, FSPDF_ANNOTTYPE_LINK)) return;

	//Set the highlighting mode of annotation.
	FSPDF_Annot_SetHighlightingMode(annot, FSPDF_ANNOT_HIGHLIGHTINGMODE_OUTLINE);

	//Insert the actions of specified index associated with the annotation.
	//Prepare the action data.
	FSPDF_ACTIONDATA data;
	data.actionType = FSPDF_ACTION_URI;
	//Prepare the URI action data.
	FSPDF_ACTIONDATA_URI setStrUrlNoAction;
	FSCRT_BStr_Init(&setStrUrlNoAction.url);
	FSCRT_BStr_Set(&setStrUrlNoAction.url, "www.google.com",(FS_DWORD)strlen("www.google.com"));
	setStrUrlNoAction.mapPosition = TRUE;
	data.actionData = (FS_LPVOID)&setStrUrlNoAction;
	
	//Insert the action data.
	FSPDF_Annot_InsertAction(annot, FSPDF_TRIGGER_ANNOT_MU, 0, &data);
	FSCRT_BStr_Clear(&setStrUrlNoAction.url);
}

//Set annotation common properties.
void CPDFAnnot::SetAnnotProperties(FSCRT_ANNOT annot, const char* sAnnotType)
{
	//Prepare the string object of the annotation name.
	FSCRT_BSTR bsAnnotName;
	FSCRT_BStr_Init(&bsAnnotName);
	FSCRT_BStr_Set(&bsAnnotName, sAnnotType, (FS_DWORD)strlen(sAnnotType));

	//Set the flags, name of annotation.
	FSPDF_Annot_SetFlags(annot, FSPDF_ANNOTFLAG_NOROTATE);
	FSPDF_Annot_SetName(annot, &bsAnnotName);

	//Set the subject, title, contents of annotation.
	FSPDF_Annot_SetSubject(annot, &bsAnnotName);
	FSPDF_Annot_SetTitle(annot, &bsAnnotName);
	FSPDF_Annot_SetContents(annot, &bsAnnotName);

	//Set the stroke color and the opacity of annotation.
	FSPDF_Annot_SetColor(annot, FALSE, 0x00FF0000);
	FSPDF_Annot_SetOpacity(annot, 1.0);

	//Set the creation time and the modification time of annotation.
	FSCRT_DATETIMEZONE dt;
	memset(&dt, 0, sizeof(FSCRT_DATETIMEZONE));
	dt.day = 7;
	dt.month = 5;
	dt.year = 2013;
	FSPDF_Annot_SetCreationDateTime(annot, &dt);
	FSPDF_Annot_SetModifiedDateTime(annot, &dt);

	//Clear the string object.
	FSCRT_BStr_Clear(&bsAnnotName);
}

/*******************************************************************************/
/*  Layout annotations to appropriate position.			                       */
/*******************************************************************************/
void CPDFAnnot::LayoutAnnots(FSCRT_ANNOT annot, const char* sAnnotType, const FS_FLOAT x, const FS_FLOAT y)
{
	//Move the annotation to point(x,y) and set the width, height to 100.
	FSCRT_RECTF rect;
	FS_RESULT ret = FSPDF_Annot_GetRect(annot, &rect);
	if(ret == FSCRT_ERRCODE_SUCCESS)
		FSDK_OutputLog("Success: The %s annotation rectangle is [%.2f %.2f %.2f %.2f].\r\n", sAnnotType, rect.left, rect.top, rect.right, rect.bottom);
	else{
		FSDK_OutputLog("Failure: Fail to get %s annotation rectangle with error code %d.\r\n", sAnnotType, ret);
	}
	rect.left = x;
	rect.bottom = y;
	rect.right = x + 100;
	rect.top = y + 100;
	ret = FSPDF_Annot_Move(annot, &rect);
	if(ret == FSCRT_ERRCODE_SUCCESS)
		FSDK_OutputLog("Success: The %s annotation move to [%.2f %.2f %.2f %.2f].\r\n", sAnnotType, rect.left, rect.top, rect.right, rect.bottom);
	else{
		FSDK_OutputLog("Failure: Fail to move %s annotation to [%.2f %.2f %.2f %.2f] with error code %d.\r\n", 
			sAnnotType, rect.left, rect.top, rect.right, rect.bottom, ret);
	}
}

/*******************************************************************************/
/*  Add annotations to PDF Page.						                       */
/*******************************************************************************/
FSCRT_ANNOT CPDFAnnot::AddAnnotToPDFPage(const FSCRT_PAGE pdfPage, const char* sAnnotType,string inputFile)
{
	//Prepare the rectangle object of annotation bounding box, in PDF page coordination.
	FSCRT_RECTF rect = {0, 100, 100, 0};
	//Prepare the string object of the annotation filter.
	FSCRT_BSTR bsAnnotType;
	FSCRT_BStr_Init(&bsAnnotType);
	FSCRT_BStr_Set(&bsAnnotType, sAnnotType, (FS_DWORD)strlen(sAnnotType));

	//Add an annotation to a specific index with specific filter.
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;
	FSCRT_ANNOT annot = NULL;
	ret = FSPDF_Annot_Add(pdfPage, &rect, &bsAnnotType, &bsAnnotType, 1, &annot);

	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		//Write the log info.
		FSDK_OutputLog("Failure: Fail to Add %s annotation with error code %d.\r\n", bsAnnotType.str, ret);
		//Clear the string object if cancel the adding process.
		FSCRT_BStr_Clear(&bsAnnotType);
		return NULL;
	}
	//Write the log info.
	FSDK_OutputLog("Success: Add %s annotation.\r\n", bsAnnotType.str);

	//Set annotation common properties.
	SetAnnotProperties(annot, sAnnotType);
	//Set Highlight annotation properties.
	SetHighlightAnnotProperties(annot, sAnnotType);
	//Set UnderLine annotation properties.
	SetUnderLineAnnotProperties(annot, sAnnotType);
	//Set Squiggly annotation properties.
	SetSquigglyAnnotProperties(annot, sAnnotType);
	//Set StrikeOut annotation properties.
	SetStrikeOutAnnotProperties(annot, sAnnotType);
	//Set Line annotation properties.
	SetLineAnnotProperties(annot, sAnnotType);
	//Set Polygon annotation properties.
	SetPolygonAnnotProperties(annot, sAnnotType);
	//Set Polyline annotation properties.
	SetPolylineAnnotProperties(annot, sAnnotType);
	//Set Ink annotation properties.
	SetInkAnnotProperties(annot, sAnnotType);
	//Set FreeText annotation properties.
	SetFreeTextAnnotProperties(annot, sAnnotType);
	//Set Link annotation properties.
	SetLinkAnnotProperties(annot, sAnnotType);
	//Set Stamp annotation properties
	SetStampAnnotProperties(annot, sAnnotType ,inputFile);

	//Regenerate the appearance of the annotation.
	ret = FSPDF_Annot_ResetAppearance(annot);
	if (FSCRT_ERRCODE_SUCCESS != ret)
		FSDK_OutputLog("Failure: Fail to reset %s annotation appearance with error code %d.\r\n", bsAnnotType.str, ret);
	else
		FSDK_OutputLog("Success: Reset %s annotation appearance.\r\n", bsAnnotType.str);

	//Clear the string object.
	FSCRT_BStr_Clear(&bsAnnotType);
	return annot;
}

/*******************************************************************************/
/*  Add annotations to PDF Document.					                       */
/*******************************************************************************/
FS_RESULT CPDFAnnot::AddAnnotToPDFFile(string fileName) //string inputFolder, string outputFolder
{
	//Define a string object to store log data.
	string strFile = m_inputFolder;
	strFile += "/";
	strFile += fileName;
	FSDK_OutputLog("\r\n====== Add annotations to %s ======\r\n", fileName.c_str());
	//Open a PDF file.
	CAutoDocument autoPdfDoc(strFile.c_str(), "rb");
	FS_RESULT ret = autoPdfDoc.LoadPDFDocument(NULL);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		return ret;
	}

	FS_INT32 nPageCount = 0;
	FS_INT32 nPageNum = 0;
	FS_INT32* nPageIndexArray = NULL;
	if (m_PageRange != "")
	{
		if (FSDK_GetPageRange(m_PageRange.c_str(), nPageIndexArray, &nPageNum))
		{
			//Count page number
			ret = FSPDF_Doc_CountPages(autoPdfDoc.GetDocument(), &nPageCount);
			if (ret != FSCRT_ERRCODE_SUCCESS)
			{
				FSDK_OutputLog("Failure: Failed to get PDF page count with error code %d.\r\n", ret);
				if (nPageIndexArray) 
					delete[] nPageIndexArray;
				nPageIndexArray = NULL;
				return ret;
			}
		}
	}

	if (nPageNum < 1) nPageNum = 1;

	//Dealing with page one by one
	for (int i=0; i<nPageNum; i++)
	{
		FS_INT32 pageIndex;
		if (m_PageRange != "" && nPageIndexArray)
		{
			if (nPageIndexArray[i] >= nPageCount || nPageIndexArray[i] < 0)
			{
				FSDK_OutputLog("Failure: Page index %d is out of valid range!\r\n", nPageIndexArray[i]);
				continue;
			}
			pageIndex = nPageIndexArray[i];
		}
		else
		{
			pageIndex = i;
		}
		//FSCRT_PAGE pdfPage = NULL;

		CAutoPDFPage autoPDFPage;
		ret  = autoPDFPage.LoadPDFPage(autoPdfDoc.GetDocument(),pageIndex);
		//ret = FSPDF_Doc_GetPage(pdfDoc, pageIndex, &pdfPage);
		if ( FSCRT_ERRCODE_SUCCESS  != ret)
		{
			if (nPageIndexArray)
			{
				delete []nPageIndexArray;
				nPageIndexArray = NULL;
			}
			return ret;
		}
		//The page should run loadAnnots before adding annotations, or the process of adding annotations cannot be executed.
		ret  = autoPDFPage.LoadAnnots();
		if (FSCRT_ERRCODE_SUCCESS == ret)
		{
			//Enumerate the type of annotations for later step of adding annotations.
			const char* annotTypes[] = {FSPDF_ANNOTTYPE_TEXT, FSPDF_ANNOTTYPE_LINK, FSPDF_ANNOTTYPE_FREETEXT, FSPDF_ANNOTTYPE_LINE, 
				FSPDF_ANNOTTYPE_SQUARE, FSPDF_ANNOTTYPE_CIRCLE, FSPDF_ANNOTTYPE_POLYGON, FSPDF_ANNOTTYPE_POLYLINE, FSPDF_ANNOTTYPE_HIGHLIGHT,
				FSPDF_ANNOTTYPE_UNDERLINE, FSPDF_ANNOTTYPE_SQUIGGLY, FSPDF_ANNOTTYPE_STRIKEOUT, FSPDF_ANNOTTYPE_STAMP, FSPDF_ANNOTTYPE_INK,
				FSPDF_ANNOTTYPE_FILEATTACHMENT};
			FS_INT32 annotNum = sizeof(annotTypes)/sizeof(annotTypes[0]);
			FSCRT_ANNOT annot = NULL;
			for (FS_INT32 i = 0; i < annotNum; i++)
			{
				//Add the specific annotation to page.
				annot = AddAnnotToPDFPage(autoPDFPage.GetPDFPage(), annotTypes[i], m_inputFolder);
				if (annot)
				{
					//Layout the annotation to appropriate position.
					LayoutAnnots(annot, annotTypes[i], (FS_FLOAT)(i%4*150), (FS_FLOAT)(i/4*100));
					//Reset appearance must be done to update the annotation.
					ret = FSPDF_Annot_ResetAppearance(annot);
					if (FSCRT_ERRCODE_SUCCESS != ret)
						FSDK_OutputLog("Failure: Fail to reset %s annotation appearance with error code %d.\r\n", annotTypes[i], ret);
					else
						FSDK_OutputLog("Success: Reset %s annotation appearance.\r\n", annotTypes[i]);
				}
			}

			//Save the modified document to the output folder
			string strOutputFile = m_outputFolder + "/" + fileName;
			ret = FSDK_SavePDFFile(autoPdfDoc.GetDocument(), strOutputFile.c_str());
		}
	}

	if (nPageIndexArray)
	{
		delete []nPageIndexArray;
		nPageIndexArray = NULL;
	}

	return ret ? FSCRT_ERRCODE_SUCCESS : FSCRT_ERRCODE_ERROR;
}

void CPDFAnnot::AddAnnotsToPDFFile(string strpdfFile)//string inputFilesPath, string outputFilesPath
{
	if (strpdfFile.size() > 0 && FSDK_CompareExtName(strpdfFile.c_str(), ".pdf"))
	{
		//Load PDF File and add various Annotations to Document will be done if success.
		AddAnnotToPDFFile(strpdfFile);
	}
}

void  CPDFAnnot::BrowerFolder(void* folder)
{
	FSDK_OutputLog("success\r\n");
	do
	{
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		if(!isDir)
		{
			AddAnnotsToPDFFile(strFile);
		}
		if (strFile.size() < 1) break;

	} while (1);
}
/*******************************************************************************/
/*  Defines the entry point for the console application.					   */
/*******************************************************************************/
int main(int argc, char* argv[])
{
	//Set the current executing file path.
	FSDK_SetExecuteFilePath(argc, argv);
// 	string inputSrcFile = "";
// 	string outputFolder = "";
	CPDFAnnot pdfAnnot;
	//Judge if to command line arguments. 
	//usage: pdfannot <srcfile> /r [pagerange] /o [destfolder]
	if (argc > 1)
	{
		pdfAnnot.m_inputSrcFile = argv[1];
		bool bArgumentValid = true;
		switch(argc)
		{
		case 2:
			break;
		case 3:
			{
				if (0 != strcmp(argv[2], "/r") && 0 != strcmp(argv[2], "/o"))
					bArgumentValid = false;
				break;
			}
		case 4:
			{//usage: pdfannot <srcfile> XXX XXX
				if (0 != strcmp(argv[2], "/r") && 0 != strcmp(argv[2], "/o"))
					bArgumentValid = false;
				else if (0 == strcmp(argv[2], "/r"))
				{
					if (0 != strcmp(argv[3], "/o") && '/'!=argv[3][0])
						pdfAnnot.m_PageRange = argv[3]; //usage: pdfannot <srcfile> /r [pagerange]
					else if (0 != strcmp(argv[3], "/o") && '/'==argv[3][0])
						bArgumentValid = false;
				}
				else if (0 == strcmp(argv[2], "/o"))
					pdfAnnot.m_outputFolder = argv[3]; //usage: pdfannot <srcfile> /o [destfolder]
				break;
			}
		case 5:
			{//usage: pdfannot <srcfile> /r XXX XXX
				if (0 != strcmp(argv[2], "/r"))
					bArgumentValid = false;
				else 
				{
					if (0 != strcmp(argv[3], "/o"))
					{
						if (0 != strcmp(argv[4], "/o"))
							bArgumentValid = false;
						else
							pdfAnnot.m_PageRange = argv[3]; //usage: pdfannot <srcfile> /r [pagerange] /o
					}
					else
						pdfAnnot.m_outputFolder = argv[4];//usage: pdfannot <srcfile> /r /o [destfolder]
				}
				break;
			}
		case 6:
			{//usage: pdfannot <srcfile> /r [pagerange] /o [destfolder]
				if (0 != strcmp(argv[2], "/r") || 0 != strcmp(argv[4], "/o"))
					bArgumentValid = false;
				else
				{
					pdfAnnot.m_PageRange = argv[3];
					pdfAnnot.m_outputFolder = argv[5];
				}
				break;
			}
		default:
			bArgumentValid = false;
		}
		if (false == bArgumentValid || !FSDK_FileExist(pdfAnnot.m_inputSrcFile.c_str()) || (pdfAnnot.m_outputFolder != "" && !FSDK_FolderExist(pdfAnnot.m_outputFolder.c_str())))
		{
			printf("\r\n");
			printf("usage: pdfannot [<srcfile> /r [pagerange] /o [destfolder]] \r\n\r\n");
			printf("<srcfile>\t: Path of a PDF file as input file.\r\n");
			printf("/r [pagerange]\t: Page range, used to specify indexes of PDF pages in input PDF\r\n ");
			printf("\t\t  file, such as \"0\", \"1-3,5\". Page index starts from 0 and\r\n\t\t  should be valid within page count. Invalid index will be\r\n\t\t  ignored. "\
				"If input page range is totally invalid, page range is\r\t\t  to be first page by default.\r\n");
			printf("/o [destfolder]\t: Path of a folder as output folder. If not set, use default\r\n\t\t  output folder.\r\n\r\n");
			return -1;
		}
	}
	// Locate the path of log file
	string logPath ;
	if (pdfAnnot.m_outputFolder != "")
	{
		pdfAnnot.m_outputFolder += "/pdfannot";
		FSDK_CreatFolder(pdfAnnot.m_outputFolder.c_str());
		logPath = pdfAnnot.m_outputFolder;
		logPath += "/log.txt";
	}
	else
	{
		pdfAnnot.m_outputFolder = FSDK_GetOutputFilesFolder("pdfannot");
		logPath = FSDK_GetLogFile("pdfannot");
	}
	
	//Open and write log
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdfannot\r\n\r\n");

	//Initialize SDK library, or no method of SDK can be used.
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		FSDK_CloseLog();
		return -1;
	}

	//Initialize PDF module, or no method of PDF module can be used.
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(pdfAnnot.m_outputFolder + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}


	//Enumerate files in specific folder and choose PDF file to load.
	//string inputFolder;
	string pdfName;
	if (pdfAnnot.m_inputSrcFile != "")
	{
		pdfName = FSDK_GetFileName(pdfAnnot.m_inputSrcFile.c_str());
		pdfAnnot.m_inputFolder = FSDK_GetCustomInputPath(pdfAnnot.m_inputSrcFile.c_str());
		FSDK_OutputLog("Input from folder: %s\r\n", pdfAnnot.m_inputFolder.c_str());
		FSDK_OutputLog("Output to folder: %s \r\n\r\n", pdfAnnot.m_outputFolder.c_str());
		pdfAnnot.AddAnnotsToPDFFile(pdfName);
	}
	else
	{
		pdfAnnot.m_inputFolder = FSDK_GetInputFilesFolder("pdfannot");
		//Open a folder to enumerate all files in it.
		void* folder = FSDK_OpenFolder(pdfAnnot.m_inputFolder.c_str(), FALSE);
		if (folder)
		{
			FSDK_OutputLog("Input from folder: %s\r\n", pdfAnnot.m_inputFolder.c_str());
			FSDK_OutputLog("Output to folder: %s \r\n\r\n", pdfAnnot.m_outputFolder.c_str());
			FSDK_OutputLog("\r\nEnumerate files in folder input_files/pdfannot: ");
			pdfAnnot.BrowerFolder(folder);
			//Close access to specific folder.
			FSDK_CloseFolder(folder);
		}
		else
		{
			pdfName = "AboutFoxit.pdf";
			pdfAnnot.m_inputFolder = FSDK_GetFixFolder();
			FSDK_OutputLog("Input from folder: %s\r\n", pdfAnnot.m_inputFolder.c_str());
			FSDK_OutputLog("Output to folder: %s \r\n\r\n", pdfAnnot.m_outputFolder.c_str());
			pdfAnnot.AddAnnotsToPDFFile(pdfName);
		}
	}


	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);
	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}
