/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is an example to simulate asynchronous access to a local PDF document. It allows reading and downloading of a PDF file to occur at the same time.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main pdfasync
#endif


//Call back functions which are realized by caller for receiving download hints
class CPDF_Asyncfile : public FSPDF_ASYNCFILEHANDLER
{
public:
	CPDF_Asyncfile(FS_LPCSTR PathName);
	~CPDF_Asyncfile();

	//Get a PDF file opened 
	static	FS_RESULT    GetFileImp(FS_LPVOID clientData, FSCRT_FILE* file);
	//Foxit SDK will call it to report whether the specified data section is available
	static FS_RESULT 		IsDataAvailImp(FS_LPVOID clientData, FS_DWORD offset, FS_DWORD size);
	//Foxit SDK will call it to add a section which is to be downloaded
	static FS_RESULT 	AddDownloadHintImp(FS_LPVOID clientData, FS_DWORD offset, FS_DWORD size);

	FS_RESULT			AsyncLoadDocment();
	FS_RESULT			ReadPDFAsync();

protected:	
	//File name which is to be open
	FS_LPCSTR				m_lpszPathName;
	//File handler 
	FSCRT_FILE				m_file;
	//Store data which have downloaded. Odd  element store offset, and even element store size.
	std::vector<FS_DWORD>	m_data;
	FSCRT_DOCUMENT			m_pdfDoc;
};

//Construction
CPDF_Asyncfile::CPDF_Asyncfile(FS_LPCSTR PathName)
{
	//Store file name
	m_lpszPathName = PathName;
	m_file = NULL;
	m_pdfDoc = NULL;
	clientData = this;
	GetFile = GetFileImp;
	IsDataAvail = IsDataAvailImp;
	AddDownloadHint = AddDownloadHintImp;
}

//Destructor
CPDF_Asyncfile::~CPDF_Asyncfile()
{
	//Release the file object
	if (m_file)
	{	
		FSCRT_File_Release(m_file);
		m_file = NULL;
	}

	if (m_pdfDoc)
	{
		FSPDF_Doc_Close(m_pdfDoc);
		m_pdfDoc = NULL;
	}
}

//Get a PDF file opened 
FS_RESULT CPDF_Asyncfile::GetFileImp(FS_LPVOID clientData, FSCRT_FILE* file)
{	
	CPDF_Asyncfile *pAsyncfile = (CPDF_Asyncfile*)clientData;

	if (!pAsyncfile)
	{
		return FSCRT_ERRCODE_ERROR;
	}

	if (!pAsyncfile->m_file)
	{
		//Open a file with read only mode
		pAsyncfile->m_file  = FSDK_OpenFile(pAsyncfile->m_lpszPathName, "rb");
	}

	*file = pAsyncfile->m_file;
    return FSCRT_ERRCODE_SUCCESS;
}

//Report whether the specified data section is available
FS_RESULT CPDF_Asyncfile::IsDataAvailImp(FS_LPVOID clientData, FS_DWORD offset, FS_DWORD size)
{
	CPDF_Asyncfile *pAsyncfile = (CPDF_Asyncfile*)clientData;

	if (!pAsyncfile)
	{
		return FSCRT_ERRCODE_ERROR;
	}

	int sizeTemp = size;
	int sizeT = 0; 
	for(unsigned int i = 0; (i < pAsyncfile->m_data.size()); i += 2) {
		if((offset >= pAsyncfile->m_data[i]) && (offset < (pAsyncfile->m_data[i] + pAsyncfile->m_data[i+1]))) {
			sizeT = pAsyncfile->m_data[i] + pAsyncfile->m_data[i+1] - offset;//If section is available return true
			if(sizeTemp <= sizeT) return FSCRT_ERRCODE_SUCCESS;
			else {
				sizeTemp = sizeTemp - sizeT;
				offset = pAsyncfile->m_data[i] + pAsyncfile->m_data[i+1]; 
				i = -2;
			}
		}
	}
	return FSCRT_ERRCODE_ERROR;//Not available return FSCRT_ERRCODE_ERROR
}

//Add a section which is to be downloaded
FS_RESULT CPDF_Asyncfile::AddDownloadHintImp(FS_LPVOID clientData, FS_DWORD offset, FS_DWORD size)
{
	CPDF_Asyncfile *pAsyncfile = (CPDF_Asyncfile*)clientData;

	if (!pAsyncfile)
	{
		return FSCRT_ERRCODE_ERROR;
	}

	unsigned int i = 0;
	for(; i < pAsyncfile->m_data.size(); i += 2){
		//If section is already downloaded, break
		if((offset >= pAsyncfile->m_data[i]) 
			&& (offset < (pAsyncfile->m_data[i] + pAsyncfile->m_data[i+1])) 
			&& (size <= (pAsyncfile->m_data[i] + pAsyncfile->m_data[i+1] - offset))) 
			break;
	}
	//If it  isn't downloaded, store offset and size to indicate that the section has downloaded 
	if (i >= pAsyncfile->m_data.size()){
		pAsyncfile->m_data.push_back(offset);
		pAsyncfile->m_data.push_back(size);
	}
    
    return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT	CPDF_Asyncfile::AsyncLoadDocment()
{
	string strLog = "";
	FS_RESULT ret = FSPDF_Doc_AsyncLoad(this, NULL, &m_pdfDoc);

	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		FSDK_OutputLog("Failure: Fail to load FDF document. And the error code %d.\r\n", ret);	
		FSDK_CHECK_LICENSE(ret);
		return TRUE;
	}

	strLog = "Success: Load FDF document.\r\n";
	FSDK_OutputStringLog(strLog.c_str(), strLog.length());
	return ret;
};

///This function is to async read a PDF file
FS_RESULT CPDF_Asyncfile::ReadPDFAsync()
{
	string fileName = FSDK_GetFileName(m_lpszPathName);
	FSDK_OutputLog("\r\n====Async Read %s ====\r\n", fileName.c_str());

	unsigned int nStartTime = FSDK_GetClock();		//Get current time

	//Load PDF file through asynchronous mode
	FS_RESULT ret = AsyncLoadDocment();

	if ( FSCRT_ERRCODE_SUCCESS !=ret)
	{
		return ret;
	}

	FS_BOOL bDocAvail = FALSE;
	while(!bDocAvail) 	//Check whether the document is available, if not continue downloading		
		 ret  = FSPDF_Doc_IsDocAvail(m_pdfDoc, &bDocAvail);
		
	if (ret == FSCRT_ERRCODE_SUCCESS && bDocAvail) //The document is available
	{
		FSDK_OutputLog("Success: The document is available.\r\n");
		FS_INT32 pageCount = 0;
		if (FSCRT_ERRCODE_SUCCESS == FSPDF_Doc_CountPages(m_pdfDoc, &pageCount))//Get page count
		{
			for(FS_INT32 index = 0; index < pageCount; index++)//Move to next page
			{
				FS_BOOL bPageAvail = FALSE;
				while(!bPageAvail)					//Check whether the page is available, if not continue downloading
					ret = FSPDF_Doc_IsPageAvail(m_pdfDoc, index, &bPageAvail);

				if (ret == FSCRT_ERRCODE_SUCCESS && bPageAvail)
				{	
					//Get the page by index
					CAutoPDFPage autoPdfPage;
					ret = autoPdfPage.LoadPDFPage(m_pdfDoc,index);
					if (FSCRT_ERRCODE_SUCCESS != ret)
					{
						continue;
					}
					//Start parsing PDF page
					autoPdfPage.ParsePage();
					
				}
			} 
		}
		//Get current time and output elapse time
		unsigned int nEndTime = FSDK_GetClock();
		FSDK_OutputLog("\tPage count: %d\r\n\telapsed time: %dms\r\n", pageCount, nEndTime - nStartTime);
	} else {
		FSDK_OutputLog("Failure: Fail to the document is unavailable.\r\n");
	}

	FSDK_OutputLog("====End Read PDF Async====\r\n");
	return ret;
}

void FSDK_ReadPDFAsyncFolder(void* folder, string inputFilesPath)
{
	FSDK_OutputLog("success.\r\n");
	
	do
	{
		//Get a file from folder
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		if (strFile.size() > 0 && !isDir)
		{
			inputFilesPath += "/" + strFile;
			CPDF_Asyncfile pAsyncFile(inputFilesPath.c_str());
			pAsyncFile.ReadPDFAsync();
		}

		//If it's an invalid file name
		if (strFile.size() < 1) break;
	}while (1);
}

int main(int argc, char* argv[])
{
	//Set executing file path
	FSDK_SetExecuteFilePath(argc, argv);

	//Open log file and output some basic info
	string logPath = FSDK_GetLogFile("pdfasync");
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdfasync\r\n\r\n");

	//Initialize Foxit SDK library
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		//Fail to initialize, output status and return
		FSDK_CloseLog();
		return -1;
	}
	//Initialize PDF module
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}
	//Get folder of input files for this example
	string inputFilesPath = FSDK_GetInputFilesFolder("pdfasync");
	string outputFilesPath = FSDK_GetOutputFilesFolder("pdfasync");

	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(outputFilesPath + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	//Open folder to enumerate files
	void* folder = FSDK_OpenFolder(inputFilesPath.c_str(), FALSE);
	if (folder)
	{
		FSDK_OutputLog("Input from folder: %s\r\n", inputFilesPath.c_str());
		FSDK_OutputLog("Output to folder: %s\r\n", outputFilesPath.c_str());
		FSDK_ReadPDFAsyncFolder(folder, inputFilesPath);
		//Close access to specific folder.
		FSDK_CloseFolder(folder);
	}
	else
	{
		inputFilesPath = FSDK_GetFixFolder();
		FSDK_OutputLog("Input from folder: %s\r\n", inputFilesPath.c_str());
		FSDK_OutputLog("Output to folder: %s\r\n", outputFilesPath.c_str());
		inputFilesPath += "/FoxitBigPreview.pdf";
		CPDF_Asyncfile pAsyncFile(inputFilesPath.c_str());
		pAsyncFile.ReadPDFAsync();
	}


	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);

	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}

