/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is an example to get bookmarks in a PDF document and edit bookmarks in PDF document, including: add, edit and delete.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main pdfbookmark
#endif

//Macro Definitions for edit bookmarks
#define Add_BookMark	0	//add a bookmark
#define Edit_BookMark	1	//edit attributes 
#define Delete_BookMark	2	//delete a bookmark

//This function is to save as a new PDF
FS_BOOL Save2PDF(FSCRT_DOCUMENT document, string fileName, string outputFilesPath, int type)
{
	//Get outputting file name
	string pdfName = string(fileName.c_str(),fileName.length()-4);
	outputFilesPath += "/";
	outputFilesPath += pdfName;

	//Different outputting file name according to edit type 
	switch (type)
	{
	case Add_BookMark:
		outputFilesPath += "_add";
		break;
	case Edit_BookMark:
		outputFilesPath += "_edit";
		break;
	case Delete_BookMark:
		outputFilesPath += "_delete";
		break;
	}
	outputFilesPath += ".pdf";
	
	//Save as a new file
	return FSDK_SavePDFFile(document, outputFilesPath.data());
}

//This function is to create a bookmark iterator and move to the root bookmark which is fictitious in a PDF  
FS_RESULT GetBookmarkRoot(FSCRT_DOCUMENT document, FSPDF_BOOKMARK_ITERATOR* bkIterator)
{
	//Create a bookmark iterator 
	FS_RESULT ret = FSPDF_Doc_CreateBookmarkIterator(document, bkIterator);
	if (ret == FSCRT_ERRCODE_SUCCESS)
	{
		//Create successfully
		FSDK_OutputLog("Success: Create an iterator to access bookmark.\r\n");
		//Move to the root bookmark
		ret = FSPDF_Bookmark_MoveToRoot(*bkIterator);
		if (ret == FSCRT_ERRCODE_SUCCESS)//Move successfully
			FSDK_OutputLog("Success: Move bookmark iterator to root.\r\n");
		else
			//Move unsuccessfully
			FSDK_OutputLog("Failure: In function FSPDF_Bookmark_MoveToRoot with error code %d.\r\n", ret);
	}
	else	//Create iterator unsuccessfully
		FSDK_OutputLog("Failure: Fail to create iterator to access bookmark with error code %d.\r\n", ret);

	return ret;
}


//This function is to preorder bookmark tree traversal 
void PreOrderBookmark(int depth, FSPDF_BOOKMARK_ITERATOR iterator, FS_RESULT nFind)
{
	depth ++;
	if (!iterator || nFind != FSCRT_ERRCODE_SUCCESS){
		//Return if not find bookmark
		depth --;
		return ;
	}

	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	if (depth < 1)//Output log the fictitious root 
		FSDK_OutputLog("Failure: In function PreOrderBookmark it is root.\r\n");
	else
	{
		//Define a ::FSPDF_BOOKMARKDATA object to store bookmark data
		FSPDF_BOOKMARKDATA bmData;
		ret = FSPDF_BookmarkData_Init(&bmData); //Initialize bookmark data
		//Get the current bookmark iterator's data
		ret = FSPDF_Bookmark_GetData(iterator, &bmData);
		if (bmData.title.len > 0 && bmData.title.str)
		{
			//Get bookmark data  successfully
			for (int i = 0; i < depth - 1; i++)
				FSDK_OutputLog("\t");//Output tabs in order to read the bookmark tree
			
			//Output bookmark's title
			FSDK_OutputLog("%s\r\n", bmData.title.str);
		}
		//Clear bookmark data
		ret = FSPDF_BookmarkData_Clear(&bmData);
	}
	//Move bookmark iterator to its first child
	ret = FSPDF_Bookmark_MoveToFirstChild(iterator);
	if(ret != FSCRT_ERRCODE_SUCCESS){
		depth --;
		return ;	//Return, if there is not a child
	}
	
	while(true)
	{
		//Get current bookmark's first child recursively
		PreOrderBookmark(depth, iterator, ret);
		//Move bookmark iterator to its next sibling if the current bookmark hasn't child
		ret = FSPDF_Bookmark_MoveToNextSibling(iterator);
		//Break, if it isn't a sibling
		if(ret != FSCRT_ERRCODE_SUCCESS) break;
	}
	
	//Traversal children and siblings finish
	if (ret != FSCRT_ERRCODE_SUCCESS) 
	{
		//Move a bookmark iterator to its parent
		ret = FSPDF_Bookmark_MoveToParent(iterator);
		if (ret != FSCRT_ERRCODE_SUCCESS){
			//Return if it isn't a parent
			depth --;
			return;
		}
	}
	depth --;
}

//This function is to get bookmarks in a PDF 
FS_RESULT FSDK_GetBookmark(string fileName, string inputFilesPath)
{
	//Begin, output log
	FSDK_OutputLog("==== Get Bookmark ====\r\n");

	unsigned int nStartTime = FSDK_GetClock();//Get current time
	inputFilesPath += '/' + fileName;
	
	//load document.
	CAutoDocument PDFDoc(inputFilesPath.c_str(), "rb");
	FS_RESULT ret = PDFDoc.LoadPDFDocument(NULL);
	if(FSCRT_ERRCODE_SUCCESS != ret)
		return ret;

	//Call function ::GetBookmarkRoot to create a bookmark iterator and move to the root bookmark
	FSPDF_BOOKMARK_ITERATOR iterBookmark = NULL;
	ret = GetBookmarkRoot(PDFDoc.GetDocument(), &iterBookmark);
	if (FSCRT_ERRCODE_SUCCESS  != ret)
	{
		if(iterBookmark)
			FSPDF_Bookmark_ReleaseIterator(iterBookmark);
		return ret;
	}

	FSDK_OutputLog("Enum bookmark items:\r\n");
	//Call function ::GetBookmarkRoot to preorder bookmark tree traversal
	PreOrderBookmark(0, iterBookmark, FSCRT_ERRCODE_SUCCESS);
	//Release a bookmark iterator
	FSPDF_Bookmark_ReleaseIterator(iterBookmark);
	//Get current time and output elapse time
	unsigned int nEndTime = FSDK_GetClock();
	FSDK_OutputLog("\tElapsed time: %dms\r\n\n", nEndTime - nStartTime);
	FSDK_OutputLog("==== End get Bookmark ====\r\n"); //End, output log to file

	return ret;
}

//This function is to edit bookmark, including: add, edit and delete
FS_RESULT FSDK_EditBookmark(string fileName, string inputFilesPath, string outputFilesPath, int type)
{
	//Begin, output log
	FSDK_OutputLog("\r\n==== Edit BookMark with type %d ====\r\n", type);
	inputFilesPath += '/' + fileName;
	CAutoDocument PDFDoc(inputFilesPath.c_str(), "rb");
	FS_RESULT ret = PDFDoc.LoadPDFDocument(NULL);
	if(FSCRT_ERRCODE_SUCCESS != ret)
		return ret;

	//Call function ::GetBookmarkRoot to create a bookmark iterator and move to the root bookmark
	FSPDF_BOOKMARK_ITERATOR iterBookmark = NULL;
	ret = GetBookmarkRoot(PDFDoc.GetDocument(), &iterBookmark);
	if (ret != FSCRT_ERRCODE_SUCCESS)//Get the root successfully
	{
		if(iterBookmark)
			FSPDF_Bookmark_ReleaseIterator(iterBookmark);
		return ret;
	}
	//Move bookmark iterator to its first child
	ret = FSPDF_Bookmark_MoveToFirstChild(iterBookmark);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: In function FSPDF_Bookmark_MoveToFirstChild with error code %d.\r\n", ret);
		FSPDF_Bookmark_ReleaseIterator(iterBookmark);
		return ret;
	}
	//Move successfully
	FSDK_OutputLog("Success: Move to first child bookmark.\r\n");
	//Define a ::FSPDF_BOOKMARKDATA object to store bookmark data
	FSPDF_BOOKMARKDATA bmData;
	ret = FSPDF_BookmarkData_Init(&bmData); //Initialize bookmark data
	switch (type)
	{
	case Add_BookMark://Add a new bookmark to the first child
		{
			//Set bookmark data
			bmData.title.str = (FS_LPSTR)"I am an added bookmark";//Title
			bmData.title.len = (FS_DWORD)strlen(bmData.title.str);
			//Color: red
			bmData.color = 0xFFFF0000;
			//Status: italic and bold
			bmData.status = FSPDF_BOOKMARKSTATUS_ITALIC | FSPDF_BOOKMARKSTATUS_BOLD;
			//Insert a new bookmark at the current bookmark position
			ret = FSPDF_Bookmark_Insert(iterBookmark, FSPDF_BOOKMARKPOS_FIRSTCHILD, &bmData);
			if (ret == FSCRT_ERRCODE_SUCCESS)
				FSDK_OutputLog("Success: Add a new bookmark to the first bookmark.\r\n");
			else
			{
				FSDK_OutputLog("Failure: Fail to add a new bookmark to the first bookmark with error code %d.\r\n", ret);
				FSDK_CHECK_LICENSE(ret);
			}
		}
		break;
	case Edit_BookMark://Edit the first child of the root
		{
			//Set bookmark data
			bmData.title.str = (FS_LPSTR)"I replaces the first child";//Title
			bmData.title.len = (FS_DWORD)strlen(bmData.title.str);
			//Color: green
			bmData.color = 0xFF00FF00;
			//Status: bold
			bmData.status = FSPDF_BOOKMARKSTATUS_BOLD;
			//Set data to the current bookmark positioned
			ret = FSPDF_Bookmark_SetData(iterBookmark, &bmData);
			if (ret == FSCRT_ERRCODE_SUCCESS)
				FSDK_OutputLog("Success: Set data of the first bookmark success.\r\n");
			else
			{
				FSDK_OutputLog("Failure: Fail to set data of the first bookmark with error code %d.\r\n", ret);
				FSDK_CHECK_LICENSE(ret);
			}
		}
		break;
	case Delete_BookMark://Delete the first child from the root
		{
			//Remove the current bookmark  of iterator
			ret = FSPDF_Bookmark_Remove(iterBookmark);
			if (ret == FSCRT_ERRCODE_SUCCESS)
				FSDK_OutputLog("Success: Remove the first bookmark success.\r\n");
			else
			{
				FSDK_OutputLog("Failure: Fail to remove the first bookmark with error code %d.\r\n", ret);
				FSDK_CHECK_LICENSE(ret);
			}
		}
		break;
	default://Unsupport type
		FSDK_OutputLog("Failure: The option type is not support.\r\n");
		break;
	}
	//Save as a new PDF accord to type
	Save2PDF(PDFDoc.GetDocument(), fileName, outputFilesPath, type);

	//Release a bookmark iterator
	FSPDF_Bookmark_ReleaseIterator(iterBookmark);

	FSDK_OutputLog("====End edit BookMark with type %d====\r\n", type);//End, output log 

	return ret;
}

void BookmarkFile(string strFile, string inputFilesPath, string outputFilesPath)
{
	if (FSDK_CompareExtName(strFile.c_str(), ".pdf"))
	{
		//Get bookmarks in a PDF
		FSDK_GetBookmark(strFile, inputFilesPath);
		//Add a bookmark to the first in a PDF
		FSDK_EditBookmark(strFile, inputFilesPath, outputFilesPath, Add_BookMark);
		//Edit the first bookmark's attributes in a PDF
		FSDK_EditBookmark(strFile, inputFilesPath, outputFilesPath, Edit_BookMark);
		//Delete the first bookmark in a PDF
		FSDK_EditBookmark(strFile, inputFilesPath, outputFilesPath, Delete_BookMark);
	}
	else
	{
		FSDK_OutputLog("Failure: Fail to invalid pdf file.\r\n");
	}
}

void BookmarkFolder(void* folder, string inputFilesPath, string outputFilesPath)
{
	do
	{
		//Get a file from folder
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		if (strFile.size() > 0 && !isDir)
			BookmarkFile(strFile, inputFilesPath, outputFilesPath);

		//If it's an invalid file name
		if (strFile.size() < 1) break;

	}while (1);
}

int main(int argc, char* argv[])
{
	//Set executing file path
	FSDK_SetExecuteFilePath(argc, argv);

	string inputSrcFile = "";
	string outputFolder = "";
	//Judge if to command line arguments. 
	//usage: pdfbookmark <srcfile> /o [destfolder]
	if (argc > 1)
	{
		inputSrcFile = argv[1];
		bool bArgumentValid = true;
		if (argc >= 3)
		{
			if (0 != strcmp(argv[2], "/o"))
				bArgumentValid = false;
			else if (argc > 3)
				outputFolder = argv[3];
		}
		if (false == bArgumentValid || !FSDK_FileExist(inputSrcFile.c_str()) || (outputFolder != "" && !FSDK_FolderExist(outputFolder.c_str())))
		{
			printf("\r\n");
			printf("usage: pdfbookmark [<srcfile> /r [pagerange] /o [destfolder]] \r\n\r\n");
			printf("<srcfile>\t: Path of a PDF file as input file.\r\n");
			printf("/r [pagerange]\t: Page range, used to specify indexes of PDF pages in input PDF\r\n ");
			printf("\t\t  file, such as \"0\", \"1-3,5\". Page index starts from 0 and\r\n\t\t  should be valid within page count. Invalid index will be\r\n\t\t  ignored. "\
				"If input page range is totally invalid, page range is\r\t\t  to be first page by default.\r\n");
			printf("/o [destfolder]\t: Path of a folder as output folder. If not set, use default\r\n\t\t  output folder.\r\n\r\n");
			return -1;
		}
	}
	// Locate the path of log file
	string logPath ;
	if (outputFolder != "")
	{
		outputFolder += "/pdfbookmark";
		FSDK_CreatFolder(outputFolder.c_str());
		logPath = outputFolder;
		logPath += "/log.txt";
	}
	else
	{
		outputFolder = FSDK_GetOutputFilesFolder("pdfbookmark");
		logPath = FSDK_GetLogFile("pdfbookmark");
	}
	
	//Open and write log
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdfbookmark\r\n\r\n");

	//Initialize Foxit SDK library
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		//Failed to initialize, output status and return
		FSDK_CloseLog();
		return -1;
	}
	
	//Initialize PDF module
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(outputFolder + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	//Get folder of input files for this example
	string inputFolder;
	string pdfName;
	if (inputSrcFile != "")
	{
		pdfName = FSDK_GetFileName(inputSrcFile.c_str());
		inputFolder = FSDK_GetCustomInputPath(inputSrcFile.c_str());
		FSDK_OutputLog("Input from folder: %s\r\n", inputFolder.c_str());
		FSDK_OutputLog("Output to folder: %s \r\n\r\n", outputFolder.c_str());
		BookmarkFile(pdfName, inputFolder, outputFolder);
	}
	else
	{
		inputFolder = FSDK_GetInputFilesFolder("pdfbookmark");
		//Open folder to enumerate files
		void* folder = FSDK_OpenFolder(inputFolder.c_str());
		if (folder)
		{
			FSDK_OutputLog("Input from folder: %s\r\n", inputFolder.c_str());
			FSDK_OutputLog("Output to folder: %s \r\n\r\n", outputFolder.c_str());
			FSDK_OutputLog("\r\nEnumerate files in folder input_files/pdfbookmark: ");
			BookmarkFolder(folder, inputFolder, outputFolder);
			//Close access to specific folder.
			FSDK_CloseFolder(folder);
		}
		else
		{
			pdfName = "Bookmark.pdf";
			inputFolder = FSDK_GetFixFolder();
			FSDK_OutputLog("Input from folder: %s\r\n", inputFolder.c_str());
			FSDK_OutputLog("Output to folder: %s \r\n\r\n", outputFolder.c_str());
			BookmarkFile(pdfName, inputFolder, outputFolder);
		}
	}


	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);

	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}

