#include "signature.h"
#ifndef WIN32 || _WIN64
#include<sys/types.h>  
#include<sys/socket.h>  
#include<netinet/in.h> 
#include <arpa/inet.h>
#include <unistd.h>
#include <netdb.h>  
#endif

#define FREE_CERT_KEY	if(pkey)\
	EVP_PKEY_free(pkey);\
	if(x509)\
	X509_free(x509);\
	if(ca)\
	sk_X509_free(ca);\

#define HANDLE_CREATE_TS_ERROR 	{\
									if (!ret)  {\
										TS_REQ_free(ts_req);\
										ts_req = NULL;\
										printf("could not create query\n");\
									}\
									TS_MSG_IMPRINT_free(msg_imprint);\
									X509_ALGOR_free(algo);\
									ASN1_OBJECT_free(policy_obj);\
									ASN1_INTEGER_free(nonce_asn1);\
									return ts_req;\
								}

/*
*This function is used to check openssl's error. 
*Usage: error_check(__FILE__, __LINE__);
*/
int error_check(char *file = NULL, int line = 0)
{
	char szErrMsg[1024] = {0};
	char *pTmp = NULL;
	unsigned long ulErr;

	ulErr = ERR_get_error(); 
	pTmp = ERR_error_string(ulErr,szErrMsg); 
	if (ulErr != 0){
		printf("file: %s:%d, reason %s\n", file, line, pTmp);
		return -1;
	}
	return 0;
}

void initialize_openssl()
{
	SSLeay_add_all_algorithms();  
	ERR_load_ERR_strings();
	ERR_load_crypto_strings();
}

static ASN1_INTEGER *create_nonce(int bits)
{
	unsigned char buf[20];
	ASN1_INTEGER *nonce = NULL;
	int len = (bits - 1) / 8 + 1;
	int i;

	/* Generating random byte sequence. */
	if (len > (int)sizeof(buf))
	{
		printf("bit count error\n");
		ASN1_INTEGER_free(nonce);
		return NULL;
	}
	if (RAND_bytes(buf, len) <= 0)
	{
		printf("can not generate random number\n");
		ASN1_INTEGER_free(nonce);
		return NULL;
	}

	/* Find the first non-zero byte and creating ASN1_INTEGER object. */
	for (i = 0; i < len && !buf[i]; ++i) ;
	if (!(nonce = ASN1_INTEGER_new()))
	{
		printf("could not create nonce\n");
		ASN1_INTEGER_free(nonce);
		return NULL;
	}

	OPENSSL_free(nonce->data);
	/* Allocate at least one byte. */
	nonce->length = len - i;
	if (!(nonce->data = (unsigned char *)OPENSSL_malloc(nonce->length + 1)))
	{
		printf("out of memory\n");
		ASN1_INTEGER_free(nonce);
		return NULL;
	}

	memcpy(nonce->data, buf + i, nonce->length);

	return nonce;
}

static TS_REQ *create_ts_query(unsigned char *digest, int len)
{
	int ret = 0;
	TS_REQ *ts_req = NULL;
	const EVP_MD *md;
	TS_MSG_IMPRINT *msg_imprint = NULL;
	X509_ALGOR *algo = NULL;
	ASN1_OBJECT *policy_obj = NULL;
	ASN1_INTEGER *nonce_asn1 = NULL;

	switch(len){
	case 20:
		md = EVP_get_digestbyname("sha1");
		break;
	case 32:
		md = EVP_get_digestbyname("sha256");
		break;
	default:
		HANDLE_CREATE_TS_ERROR;
	}

	/* Creating request object. */
	if (!(ts_req = TS_REQ_new()))
		HANDLE_CREATE_TS_ERROR;

	/* Setting version. */
	if (!TS_REQ_set_version(ts_req, 1))
		HANDLE_CREATE_TS_ERROR;

	/* Creating and adding MSG_IMPRINT object. */
	if (!(msg_imprint = TS_MSG_IMPRINT_new()))
		HANDLE_CREATE_TS_ERROR;

	/* Adding algorithm. */
	if (!(algo = X509_ALGOR_new()))
		HANDLE_CREATE_TS_ERROR;
	if (!(algo->algorithm = OBJ_nid2obj(EVP_MD_type(md))))
		HANDLE_CREATE_TS_ERROR;
	if (!(algo->parameter = ASN1_TYPE_new()))
		HANDLE_CREATE_TS_ERROR;
	algo->parameter->type = V_ASN1_NULL;
	if (!TS_MSG_IMPRINT_set_algo(msg_imprint, algo))
		HANDLE_CREATE_TS_ERROR;

	if (!TS_MSG_IMPRINT_set_msg(msg_imprint, digest, len))
		HANDLE_CREATE_TS_ERROR;

	if (!TS_REQ_set_msg_imprint(ts_req, msg_imprint))
		HANDLE_CREATE_TS_ERROR;

	/* Setting nonce if requested. */
	if (!(nonce_asn1 = create_nonce(64)))
		HANDLE_CREATE_TS_ERROR;
	if (nonce_asn1 && !TS_REQ_set_nonce(ts_req, nonce_asn1))
		HANDLE_CREATE_TS_ERROR;

	/* Setting certificate request flag if requested. */
	if (!TS_REQ_set_cert_req(ts_req, 1))
		HANDLE_CREATE_TS_ERROR;

	ret = 1;
	HANDLE_CREATE_TS_ERROR;
}

static long Get_TS_REP(unsigned char *md, int md_len, TIMESTAMP_PARAM *TSParam, unsigned char **pRet)
{
	TS_REQ *request = create_ts_query(md, md_len);
	int len = i2d_TS_REQ(request, NULL);
	unsigned char *tsReq = (unsigned char *)OPENSSL_malloc(len);

	unsigned char *p=tsReq;
	len = i2d_TS_REQ(request, (unsigned char **)&p);
	TS_REQ_free(request);

	char SendBuffer[1024] =""; 

	sprintf(SendBuffer,	"GET %s HTTP/1.1\r\n"
						"Accept: */*\r\n"
						"Content-Type: application/timestamp-query\r\n"
						"Content-Length: %d\r\n"
						"Character-Encoding: binary\r\n"
						"User-Agent: PPKHandler\r\n"
						"Host: %s\r\n"
						"Connection: Keep-Alive\r\n"
						"Cache-Control: no-cache\r\n"
						"Authorization: Basic %s\r\n"
						"\r\n", 
						TSParam->tsaReqUrl.str,len, TSParam->tsaHost.str,TSParam->tsaUserNPswd.str );

	unsigned char *totalBuf = (unsigned char *)malloc(len+strlen(SendBuffer));

	memcpy(totalBuf, SendBuffer, strlen(SendBuffer));
	memcpy(totalBuf+strlen(SendBuffer), tsReq, len);

	OPENSSL_free(tsReq);

#ifdef _WIN32
	WSADATA  Ws;
	WSAStartup(MAKEWORD(2,2), &Ws);
	SOCKET CientSocket;
#else
	int CientSocket;
#endif
	struct sockaddr_in ServerAddr;
	int Ret = 0;
	int AddrLen = 0;


	CientSocket = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);

#ifdef _WIN32
	HOSTENT * iphost;
#else
	struct hostent *iphost;
#endif
	char serverIP[20]="";

	if((iphost = gethostbyname(TSParam->tsaHost.str)) != NULL)
	{
		int i = 0;
		while(iphost->h_addr_list[i])
		{
			memcpy(&serverIP, inet_ntoa(*((struct in_addr *)iphost->h_addr_list[i])), 20);
			i++;
		}
	}

	ServerAddr.sin_family = AF_INET;
	ServerAddr.sin_addr.s_addr = inet_addr(serverIP);
	ServerAddr.sin_port = htons(TSParam->port);
	memset(ServerAddr.sin_zero, 0x00, 8);
	connect(CientSocket,(struct sockaddr*)&ServerAddr, sizeof(ServerAddr));
	int sendLen = send(CientSocket, (const char *)totalBuf, len+(int)strlen(SendBuffer), 0);

	free(totalBuf);

	char recvBuf[8096]="";
	int revtotalLen = 0, recTSL = 0;
	int revLen = recv(CientSocket,recvBuf, 8096, 0);
	int nTSRepPos = 0;
	int tsrepL =0;
	if (revLen == 0)
	{
		printf("\r\ntsa server unreachable.\r\n");
#ifdef _WIN32
		closesocket(CientSocket);
		WSACleanup();
#else
		close(CientSocket);
#endif
		return 0;
	}
	revtotalLen+=revLen;

	//get the ts reponse length
	string strRecv = recvBuf;
	string rspCode = strRecv.substr(9,3);
	if (rspCode != "200")
	{
		printf("\r\n tsa server refuse request.\r\n%s\r\n", recvBuf);
#ifdef _WIN32
		closesocket(CientSocket);
		WSACleanup();
#else
		close(CientSocket);
#endif
		return 0;
	}
	nTSRepPos = (int)(strRecv.find("Content-Length") + strlen("Content-Length") +2);
	
	while (strRecv.at(nTSRepPos) > 47 && strRecv.at(nTSRepPos) < 58)
	{
		tsrepL = tsrepL*10 + strRecv.at(nTSRepPos) - 48;
		nTSRepPos+=1;
	}

	while (recTSL < tsrepL && revtotalLen < tsrepL)
	{
		printf("retry, len:%d\n",revLen);
		revLen = recv(CientSocket,recvBuf+revtotalLen, 8096-revtotalLen, 0);
		recTSL += revLen;
		revtotalLen+=revLen;
	}
	*pRet = (unsigned char *)OPENSSL_malloc(tsrepL);
	memcpy(*pRet, recvBuf +revtotalLen- tsrepL, tsrepL);

#ifdef _WIN32
	closesocket(CientSocket);
	WSACleanup();
#else
	close(CientSocket);
#endif
	return tsrepL;
}

static int append_tsp_token(PKCS7_SIGNER_INFO *sinfo, unsigned char *ts_rep, long tsrepL)
{
	unsigned char *p = ts_rep;
	TS_RESP *tsp = d2i_TS_RESP(NULL, (const unsigned char**)&p, tsrepL);
	if (tsp != NULL)
	{
		PKCS7* token = tsp->token;
		if (!PKCS7_type_is_signed(token))
		{
			printf("Error in timestamp token: not signed!\n");
			return 1;
		}

		int p7_len = i2d_PKCS7(token, NULL); 
		unsigned char *p7_der = (unsigned char *)OPENSSL_malloc(p7_len); 
		unsigned char *p = p7_der; 
		i2d_PKCS7(token, &p);

		if (sinfo)
		{
			//Add timestamp token to the PKCS7 signature object
			ASN1_STRING *value = ASN1_STRING_new();
			ASN1_STRING_set(value, p7_der, p7_len);
			PKCS7_add_attribute(sinfo, NID_id_smime_aa_timeStampToken,	V_ASN1_SEQUENCE, value);
			OPENSSL_free(p7_der);
		}
	}
	else
	{
		printf("Error decoding timestamp token!\n");
		return 1;
	}

	return 0;
}

static int parseP12File(FSCRT_BSTR *path, FSCRT_BSTR* pwd, EVP_PKEY** pkey, X509** x509, STACK_OF(X509)** ca)
{
	FILE* fp = NULL; 
	if (!(fp = fopen(path->str, "rb")))
	{
		fprintf(stderr, "Error opening file %s\n", path->str);
		return 0;
	}

	PKCS12* p12 = d2i_PKCS12_fp(fp, NULL); 
	fclose (fp);      
	if (!p12)
	{
		fprintf(stderr, "Error reading PKCS#12 file\n");
		ERR_print_errors_fp (stderr);  
		return 0;
	}

	if (!PKCS12_parse(p12, pwd->str, pkey, x509, ca)) {  
		fprintf(stderr, "Error parsing PKCS#12 file\n");
		return 0; 
	}   

	PKCS12_free(p12); 
	if (!pkey) {   
		ERR_print_errors_fp (stderr);  
		return 0;
	}
	return 1;
}

unsigned char* PKCS7_Sign(TIMESTAMP_PARAM *tsParam, const FSCRT_BSTR* plainText,int& retSize, FS_BOOL onlyTS)
{
	PKCS7* p7;
	TS_RESP *tsp = NULL;
	if (onlyTS)
	{
		unsigned char *pTSRep = NULL;
		long lTSRepLength = Get_TS_REP((unsigned char *)plainText->str,plainText->len, tsParam, &pTSRep);
		if (lTSRepLength == 0)
			return NULL;
		tsp = d2i_TS_RESP(NULL, (const unsigned char**)&pTSRep, lTSRepLength);

		p7 = tsp->token;
	}
	else{
		int err = 0;

		EVP_PKEY* pkey = NULL;       
		X509* x509 = NULL;  
		STACK_OF(X509)* ca = NULL; 

		if(!parseP12File(&tsParam->certFile, &tsParam->certPwd, &pkey, &x509, &ca))
			return NULL;

		p7 = PKCS7_new();
		PKCS7_set_type(p7, NID_pkcs7_signed);
		PKCS7_content_new(p7, NID_pkcs7_data);	

		//Application should not judge the sign algorithm with the content's length.
		//Here, just for convenient;
		if (plainText->len > 32)
			PKCS7_ctrl(p7, PKCS7_OP_SET_DETACHED_SIGNATURE, 1, NULL);

		PKCS7_SIGNER_INFO* info = PKCS7_add_signature(p7, x509, pkey, EVP_sha1());
		PKCS7_add_certificate(p7, x509);

		for (int i = 0; i< sk_num(CHECKED_STACK_OF(X509,ca)); i++)
			PKCS7_add_certificate(p7, (X509*)sk_value(CHECKED_STACK_OF(X509,ca), i));

		//Set source data to BIO.
		BIO* p7bio = PKCS7_dataInit(p7, NULL);
		BIO_write(p7bio, plainText->str, plainText->len);
		PKCS7_dataFinal(p7, p7bio); 

		if (tsParam->TSType)
		{
			//append the time stamp token to pkcs7 structure.
			unsigned char md256[32] = "";
			SHA256(info->enc_digest->data, info->enc_digest->length, md256);

			unsigned char *pTSRep = NULL;
			long lTSRepLength = Get_TS_REP(md256,32, tsParam, &pTSRep);
			if (lTSRepLength != 0)
				append_tsp_token(info,pTSRep,lTSRepLength );
			OPENSSL_free(pTSRep);
		}

		FREE_CERT_KEY;
		BIO_free_all(p7bio); 
	}
	//Get Signed data.
	unsigned char* der = NULL;  
	unsigned char* derTmp = NULL;  
	unsigned long derlen = i2d_PKCS7(p7, NULL);  
	der = (unsigned char*)malloc(derlen);  
	memset(der, 0, derlen);
	derTmp = der;  
	i2d_PKCS7(p7, &derTmp);
	if (tsp)
	{
		TS_RESP_free(tsp);
	}
	else
		PKCS7_free(p7);   


	retSize = derlen;
	return (unsigned char*)der;
}

FS_BOOL PKCS7_VerifySig(const FSCRT_BSTR* signedData, const FSCRT_BSTR* bstrPlainText, FS_BOOL onlyTs)
{
	FS_BOOL bSigAppr = FALSE;
	int length = 0;
	unsigned char *p = NULL;

	//Retain PKCS7 object from signed data. 
	BIO* vin = BIO_new_mem_buf(signedData->str, signedData->len);
	PKCS7* p7 = d2i_PKCS7_bio(vin, NULL);  

	STACK_OF(PKCS7_SIGNER_INFO) *sk = PKCS7_get_signer_info(p7);
	int signCount = sk_PKCS7_SIGNER_INFO_num(sk);

	if(onlyTs)
	{
		BIO* p7bio = PKCS7_dataDecode(p7, 0, 0, 0); 
		unsigned char src[1024]="";
		length =BIO_read(p7bio,src,1024); 

		p=src;
		TS_TST_INFO *req = d2i_TS_TST_INFO (NULL,(const unsigned char **)&p, length);
		//if time stamp is correct, time string is in req->time->data.
		if (!memcmp(bstrPlainText->str, req->msg_imprint->hashed_msg->data, bstrPlainText->len))
			bSigAppr = TRUE;
		BIO_free(p7bio);
	}
	else
	{
		for(int i=0;i<signCount; i++)
		{
			PKCS7_SIGNER_INFO *signInfo = sk_PKCS7_SIGNER_INFO_value(sk,i);

			ASN1_TYPE *tstoken = PKCS7_get_attribute(signInfo, NID_id_smime_aa_timeStampToken);
			if (tstoken)//verify the time stamp token.
			{
				FS_BOOL bTSAvail = FALSE;
				ASN1_STRING *set = (ASN1_STRING *)(tstoken->value.set);
				length = ASN1_STRING_length(set);
				unsigned char * tstokeng = ASN1_STRING_data(set);

				p = tstokeng;
				PKCS7* tsp7 = d2i_PKCS7(NULL, (const unsigned char**)&p, length);

				BIO* p7bio = PKCS7_dataDecode(tsp7, 0, 0, 0); 
				unsigned char src[1024]="";
				length =BIO_read(p7bio,src,1024); 

				p=src;
				TS_TST_INFO *req = d2i_TS_TST_INFO (NULL,(const unsigned char **)&p, length);
				//if time stamp is correct, time string is in req->time->data.
				if (req->msg_imprint->hashed_msg->length == 20)
				{
					unsigned char md[20];
					SHA1(signInfo->enc_digest->data,signInfo->enc_digest->length,md);
					if (!memcmp(md, req->msg_imprint->hashed_msg->data, 20))
						bTSAvail = TRUE;
				}
				else if (req->msg_imprint->hashed_msg->length == 32)
				{
					unsigned char md[32];
					SHA256(signInfo->enc_digest->data,signInfo->enc_digest->length,md);
					if (!memcmp(md, req->msg_imprint->hashed_msg->data, 32))
						bTSAvail = TRUE;
				}
				BIO_free(p7bio);
				PKCS7_free(tsp7); 
			}

			BIO *p7bio = BIO_new_mem_buf(bstrPlainText->str, bstrPlainText->len);;
			X509 *x509= PKCS7_cert_from_signer_info(p7,signInfo);
			int err = PKCS7_verify(p7, NULL, NULL,p7bio, NULL, PKCS7_NOVERIFY); 
			if(err == 1)
			{
				bSigAppr = TRUE;
			}
			BIO_free(p7bio);
		}

		PKCS7_free(p7);            
		BIO_free(vin);    
	}

	return bSigAppr;  
}
