/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is an example to render all PDF Layers of a PDF document.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main pdflayer
#endif

//Render page to image
FS_RESULT PageToImage(FSCRT_PAGE page, FSPDF_LAYERCONTEXT viewContext, FSCRT_BITMAP* bmp)
{
	//Get the given page's size.
	FS_FLOAT width = 0, height = 0;
	FS_INT32 nBitmapType = FSCRT_BITMAPFORMAT_24BPP_BGR;
	FS_ARGB colorToFill = FSCRT_ARGB_Encode(0xff, 0xff, 0xff, 0xff);
	FSPDF_Page_GetSize(page, &width, &height);
	
	FS_BOOL hasTrans = FALSE;
	FSPDF_Page_HasTransparency(page, &hasTrans);
	if(hasTrans)
	{
		FSDK_OutputLog("Success:Page has transparency.\r\n");
		nBitmapType = FSCRT_BITMAPFORMAT_32BPP_BGRA;
		colorToFill = FSCRT_ARGB_Encode(0x00, 0xff, 0xff, 0xff);
	}

	//Get a bmp handler to hold bmp data from rendering progress.
	FSCRT_BITMAP bitmap = NULL;
	FS_RESULT ret = FSCRT_Bitmap_Create((FS_INT32)width, (FS_INT32)height, nBitmapType, NULL, 0, &bitmap);
	if (ret != FSCRT_ERRCODE_SUCCESS) 
		return ret;

	FSDK_OutputLog("Success:Create bitmap.\r\n");

	//Set rect area and color space of bitmap.
	FSCRT_RECT rect = {0, 0, (FS_INT32)width, (FS_INT32)height};
	FSCRT_Bitmap_FillRect(bitmap, colorToFill, &rect);

	//Create a renderer on the given bitmap.
	FSCRT_RENDERER renderer;
	ret = FSCRT_Renderer_CreateOnBitmap(bitmap, &renderer);
	if (ret != FSCRT_ERRCODE_SUCCESS) {
		FSCRT_Bitmap_Release(bitmap);
		return ret;
	}

	FSDK_OutputLog("Success:Create a rendercontext.\r\n");
	//Create a render context and render a page to get a bmp later.
	FSPDF_RENDERCONTEXT rendercontext;
	ret = FSPDF_RenderContext_Create(&rendercontext);
	if (ret != FSCRT_ERRCODE_SUCCESS) {
		FSCRT_Renderer_Release(renderer);
		FSCRT_Bitmap_Release(bitmap);
		return ret;
	}
	//Get the page's matrix.
	FSCRT_MATRIX mt;
	FSPDF_Page_GetMatrix(page, 0, 0, (FS_INT32)width, (FS_INT32)height, 0, &mt);

	//Set the matrix of the given render context.
	FSPDF_RenderContext_SetMatrix(rendercontext, &mt);

	//Set layercontext to rendercontext.
	ret = FSPDF_RenderContext_SetLayerContext(rendercontext, viewContext);
	if (ret != FSCRT_ERRCODE_SUCCESS)  {
		FSPDF_RenderContext_Release(rendercontext);
		FSCRT_Renderer_Release(renderer);
		FSCRT_Bitmap_Release(bitmap);		
		return ret;
	}
	FSDK_OutputLog("Success:Set layercontext to rendercontext.\r\n");

	//Start to render with the given render context, renderer, page to get the render progress.
	FSCRT_PROGRESS renderProgress = NULL;
	ret = FSPDF_RenderContext_StartPage(rendercontext, renderer, page,FSPDF_PAGERENDERFLAG_NORMAL, &renderProgress);
	if (ret == FSCRT_ERRCODE_SUCCESS)
	{
		ret = FSCRT_Progress_Continue(renderProgress, NULL);
		*bmp = bitmap;
		if (ret == FSCRT_ERRCODE_FINISHED)
		{
			FSDK_OutputLog("Success:Render page.\r\n");
			ret = FSCRT_ERRCODE_SUCCESS;
		}
		else
			FSDK_OutputLog("Failure:Render page.\r\n");
	}
	else
	{
		FSCRT_Bitmap_Release(bitmap);	
		FSDK_OutputLog("Failure:Render page.\r\n");
	}

	//Release render progress 
	FSCRT_Progress_Release(renderProgress);
	//Release render
	FSCRT_Renderer_Release(renderer);
	//Release render context
	FSPDF_RenderContext_Release(rendercontext);

	return ret;
}

//Render each page to image and save the image
void RenderLayerContext(FSCRT_DOCUMENT pdfDoc, FSPDF_LAYERCONTEXT viewContext, string strFileName)
{
	//Locate the output folder
	string strFile = FSDK_GetOutputFilesFolder("pdflayer");
	strFile += "/";

	//strFile = filename - ".pdf" 
	strFile += string(strFileName.c_str(), strFileName.length() - 4);

	//Get count of PDF pages
	FS_INT32 pageCount = 0;
	FS_RESULT ret = FSPDF_Doc_CountPages(pdfDoc, &pageCount);
	if (ret == FSCRT_ERRCODE_SUCCESS) {
		FSDK_OutputLog("Success:Count the pages = %d.\r\n",pageCount);
	}
	else
		FSDK_OutputLog("Failure:Count the pages.\r\n");

	for (FS_INT32 i = 0; i < pageCount; i++) {

		//Get each page
		FSCRT_PAGE page = NULL;
		ret = FSPDF_Doc_GetPage(pdfDoc, i, &page);
		if (ret == FSCRT_ERRCODE_SUCCESS) {
			FSDK_OutputLog("Success:Get the page[%d].\r\n",i);
		}
		else
		{
			FSDK_OutputLog("Failure:Get the page[%d].\r\n",i);
			continue;
		}

		//Before loading a text page from a PDF page, the PDF page should be parsed at first.
		FSCRT_PROGRESS progress=NULL;
		ret = FSPDF_Page_StartParse(page, FSPDF_PAGEPARSEFLAG_NORMAL, &progress);
		//Continue to parse
		if (ret == FSCRT_ERRCODE_SUCCESS) 
			ret = FSCRT_Progress_Continue(progress, NULL);
		if (ret == FSCRT_ERRCODE_FINISHED) {
			FSDK_OutputLog("Success:Parse the page[%d].\r\n",i);
		}
		else
		{
			FSDK_OutputLog("Failure:Parse the page[%d].\r\n",i);
			continue;
		}
		//Release progress resource	
	    FSCRT_Progress_Release(progress);

		//Render page to image
		FSCRT_BITMAP bmp = NULL;
		ret = PageToImage(page, viewContext, &bmp);
		if (ret == FSCRT_ERRCODE_SUCCESS) {
			FSDK_OutputLog("Success:Render page[%d] to image.\r\n",i);
			//Save image file
			string fileName = strFile + "_";
			char buf[126];
			sprintf(buf,"%d",i);
			fileName += buf;
			fileName += ".bmp";
			ret = FSDK_SaveImageFile(bmp, FSCRT_IMAGETYPE_BMP, fileName.c_str());
			if (ret == FSCRT_ERRCODE_SUCCESS) {
				FSDK_OutputLog("Success:Save page[%d] to image file.\r\n",i);
			}
			else{
				FSDK_OutputLog("Failure:Save page[%d] to image file.\r\n",i);
			}

			//Destroy a bitmap and release all resources associated.
			FSCRT_Bitmap_Release(bmp);	
		}
		else
			FSDK_OutputLog("Failure:Render page[%d] to image.\r\n",i);

		//Release all page contents and relative resources.
		FSPDF_Page_Clear(page);
	}
}

//This function is to traverse layer tree and set the opposite visible state of every  PDF layer 
void TraverseLayer(FSPDF_LAYERCONTEXT context, FSPDF_LAYERNODE* pLayers, int depth)
{
	//Output log of depth of layer tree
	++depth;
	FS_RESULT nRet = FSCRT_ERRCODE_ERROR;
	if (pLayers == NULL)
	{
		//Return if layer is NULL
		--depth;
		return;
	}
	//The layer node is a PDF layer
	if (pLayers->children == NULL && pLayers->count == 0) 
	{
		for (int j = 0; j < depth + 1; j++)
		{
			FSDK_OutputLog("..");//Output tabs in order to read the layer tree
		}
		//Define a ::FSCRT_BSTR object to store layer name
		FSCRT_BSTR layerName;
		nRet = FSCRT_BStr_Init(&layerName);//Initialize layer name
		nRet = FSPDF_Layer_GetName(pLayers->layer, &layerName);//Get the layer name
		if (FSCRT_ERRCODE_SUCCESS == nRet)
		{
			//Get layer name  successfully and output layer name
			string name(layerName.str, layerName.len);
			FSDK_OutputStringLog(name.c_str(), (int)name.length());

			//Get layer visible state and output visible state
			FS_BOOL isVisible = FALSE;
			nRet = FSPDF_LayerContext_IsVisible(context, pLayers->layer, &isVisible);
			if(isVisible)
				FSDK_OutputLog("[*]\r\n");
			else
				FSDK_OutputLog("[ ]\r\n");
			//Set the opposite visible state of getting 
			nRet = FSPDF_LayerContext_SetVisible(context, pLayers->layer, !isVisible);
		}
		//Clear layer name
		nRet = FSCRT_BStr_Clear(&layerName);
	}
	else //The layer node is a PDF layer group
	{
		for (int j = 0; j < depth + 1; j++)
		{
			FSDK_OutputLog("..");//Output tabs in order to read the layer tree
		}
		//Define a ::FSCRT_BSTR object to store group name and output
		string name(pLayers->name.str, pLayers->name.len);
		FSDK_OutputStringLog(name.c_str(), (int)name.length());
		FSDK_OutputLog("\r\n");

		//traverse children layer tree recursively
		int cnt = pLayers->count;
		for (int i = 0; i < cnt; i++)
		{
			//Get the child of special index
			FSPDF_LAYERNODE tempLayer = pLayers->children[i];
			TraverseLayer(context, &tempLayer, depth);
		}
	}
	--depth;
}

///This function is to get page label information
FS_RESULT FSDK_RenderLayerContext(string strFile, string inputFilesPath)
{
	//Begin, output log 
	FSDK_OutputLog("\r\n====begin Render Layer Context====\r\n");

	//Get inputting file name and output log 
	FSDK_OutputLog("**");
	FSDK_OutputStringLog(strFile.c_str(), (int)strFile.length());
	FSDK_OutputLog("**\r\n");
	
	unsigned int nStartTime = FSDK_GetClock();//Get current time
	//Open file
	FSCRT_FILE file = FSDK_OpenFile((inputFilesPath + "/" + strFile).c_str(), "rb");
	if (!file)
	{
		//Failure, output log and return
		FSDK_OutputLog("Failure:Open file.\r\n");
		return FSCRT_ERRCODE_ERROR;
	}
	//Open successfully
	FSDK_OutputLog("Success:Open file.\r\n");

	//Load PDF file
	FSCRT_DOCUMENT pdfDoc = NULL;
	FS_RESULT ret = FSPDF_Doc_StartLoad(file, NULL, &pdfDoc, NULL);
	if (ret == FSCRT_ERRCODE_SUCCESS)
	{
		//Load successfully
		FSDK_OutputLog("Success:Load PDF document.\r\n");

		//Create a view layer context
		FSPDF_LAYERCONTEXT viewContext = NULL;
		ret = FSPDF_LayerContext_Create(pdfDoc, FSPDF_LAYERUSAGETYPE_VIEW, &viewContext);
		if (ret == FSCRT_ERRCODE_SUCCESS)
		{
			//Define a ::FSPDF_LAYERNODE object to store all PDF Layers
			FSPDF_LAYERNODE layers;
			ret = FSPDF_LayerNode_Init(&layers);//Initialize layer node
			ret = FSPDF_Doc_EnumLayers(pdfDoc, &layers);//Enum all PDF Layers.
			if (ret == FSCRT_ERRCODE_SUCCESS)
			{
				//Call function ::TraverseLayer to traverse layer tree and set the opposite visible state of every  PDF layer 
				TraverseLayer(viewContext, &layers, -1);

				//Render each page to image and save the image
				RenderLayerContext(pdfDoc, viewContext, strFile);

				//Clear layer node
				ret = FSPDF_LayerNode_Clear(&layers);
			}
			
		//release PDF Layer Context
		ret = FSPDF_LayerContext_Release(viewContext);
		}
		//Close PDF document
		FSPDF_Doc_Close(pdfDoc);

		//Get current time and output elapse time
		unsigned int nEndTime = FSDK_GetClock();
		FSDK_OutputLog("\r\nelapsed time: %dms\r\n", nEndTime - nStartTime);
	}
	else	//Fails when load PDF file
		FSDK_OutputLog("Failure:Load PDF document.\r\n");
	
	FSCRT_File_Release(file);//Release file object
	//End, output log 
	FSDK_OutputLog("\r\n====End Render Layer Context====\r\n");

	return ret;
}

void FSDK_GetPageLayerFolder(void* folder, string inputFilesPath)
{
	do
	{
		//Get a file from folder
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);

		if(!isDir && FSDK_CompareExtName(strFile.c_str(), ".pdf")) 
			FSDK_RenderLayerContext(strFile, inputFilesPath);
		
		//If it's an invalid file name
		if (strFile.size() < 1) break;
	} while (1);
}

int main(int argc, char* argv[])
{
	//Set executing file path
	FSDK_SetExecuteFilePath(argc, argv);

	//Open log file and output some basic info
	string logPath = FSDK_GetLogFile("pdflayer");
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdflayer.\r\n");
	FSDK_OutputLog("Input from: input_files folder.\r\n");
	FSDK_OutputLog("Output to: output_files/pdflayer.\r\n\r\n");

	//Initialize Foxit SDK library
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		//Fail to initialize, output status and return
		FSDK_CloseLog();
		return -1;
	}

	//Initialize PDF module
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}
	string outputFolderPath = FSDK_GetOutputFilesFolder("pdflayer");
	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(outputFolderPath + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	//Get folder of input files for this example
	string inputFilesPath = FSDK_GetInputFilesFolder("pdflayer");
	//Open folder to enumerate files
	void* folder = FSDK_OpenFolder(inputFilesPath.c_str());
	if (folder)
	{
		FSDK_OutputLog("Enumerate files in folder input_files/pdflayer: \r\n");
		FSDK_GetPageLayerFolder(folder, inputFilesPath);
		//Close access to specific folder.
		FSDK_CloseFolder(folder);
	}
	else
	{
		string inputFilesPath = FSDK_GetFixFolder();
		string strFile = "PDFLayer_3c.pdf";
		FSDK_RenderLayerContext(strFile, inputFilesPath);
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);
	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}

