/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a source to demonstrate printing a PDF page.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"
#include "math.h"

string g_PageRange = "";
/*******************************************************************************/
/*  Print the PDF page procedure.		    				                   */
/*******************************************************************************/

//Render the page by the printer device context.
void renderPage(HDC hDC, RECT rect, FSCRT_PAGE page)
{
	//Define the log string.
	string strLog("\r\nRender the Page:\r\n");

	//Calculate the size of printer canvas. The print page can be assumed as a canvas.
	FS_INT32 width = rect.right - rect.left;
	FS_INT32 height = rect.bottom - rect.top;

	//Clear the canvas with the white color.
	HBRUSH hBrush =	CreateSolidBrush(RGB(0xff, 0xff, 0xff));
	FillRect(hDC, &rect, hBrush);
	DeleteObject(hBrush);

	//Get page transformation matrix.
	FSCRT_MATRIX matrix;
	FSPDF_Page_GetMatrix(page, rect.left, rect.top, width, height, 0, &matrix);

	//Create a renderer on the printer device context.
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;
	FSCRT_RENDERER render;
	ret = FSCRT_Renderer_CreateOnWindowsDC(hDC, &render);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		//Unknown error occurs, release resource and return.
		strLog += "Fails when create render\r\n";
		FSDK_OutputStringLog(strLog.c_str(), (int)strLog.length());
		return;
	}

	FSCRT_RECT clip_rect;
	clip_rect.left = rect.left;
	clip_rect.top = rect.top;
	clip_rect.right = rect.right;
	clip_rect.bottom = rect.bottom;
	FSCRT_Renderer_SetClipRect(render, &clip_rect);


	//Create a PDF rendering context. The PDF rendering context is designed specially to ensure rendering PDF contents on rendering engine.
	FSPDF_RENDERCONTEXT context;
	ret = FSPDF_RenderContext_Create(&context);
	if (ret != FSCRT_ERRCODE_SUCCESS) {
		//Unknown error occurs, release resource and return.
		strLog += "Fails when create rendering context\r\n";
		FSDK_OutputStringLog(strLog.c_str(), (int)strLog.length());
		FSCRT_Renderer_Release(render);
		return;
	}
	//Set a transformation matrix to a rendering context.
	FSPDF_RenderContext_SetMatrix(context, &matrix);

	//Set flags to a rendering context.
	FSPDF_RenderContext_SetFlags(context, FSPDF_RENDERCONTEXTFLAG_ANNOT);

	//Start rendering a PDF page in a renderer with a PDF rendering context.
	FSCRT_PROGRESS progress;
	ret = FSPDF_RenderContext_StartPage(context, render, page, FSPDF_PAGERENDERFLAG_NORMAL, &progress);
	if (ret != FSCRT_ERRCODE_SUCCESS) {
		//Unknown error occurs, release resource and return.
		strLog += "Fails when start rendering the page\r\n";
		FSDK_OutputStringLog(strLog.c_str(), (int)strLog.length());
		FSPDF_RenderContext_Release(context);
		FSCRT_Renderer_Release(render);
		return;
	}

	//Continue a progressive process.
	ret = FSCRT_ERRCODE_TOBECONTINUED;
	while (ret == FSCRT_ERRCODE_TOBECONTINUED) {
		ret = FSCRT_Progress_Continue(progress, NULL);
	}
	if (ret != FSCRT_ERRCODE_FINISHED) {
		//Unknown error occurs, release resource and return.
		strLog += "Fails when continue rendering the page\r\n";
		FSDK_OutputStringLog(strLog.c_str(), (int)strLog.length());
	}

	//Release the current progressive process.
	FSCRT_Progress_Release(progress);

	//Render the page success.
	strLog += "success\r\n";
	//Output the log string.
	FSDK_OutputStringLog(strLog.c_str(), (int)strLog.length());
	//Release rendering context.
	FSPDF_RenderContext_Release(context);
	//Release renderer.
	FSCRT_Renderer_Release(render);
}

//Prepare the printer device context for printing.
FS_RESULT printPage(FSCRT_PAGE page)
{
	//The Page is not needed to print when its size is nearly zero.
	FS_FLOAT fPageWidth(0), fPageHeight(0);
	FSPDF_Page_GetSize(page, &fPageWidth, &fPageHeight);
	if (fabsf(fPageWidth) < 1E-5 || fabsf(fPageHeight) < 1E-5)
		return FSCRT_ERRCODE_ERROR;

	//Define the log string.
	string strLog("\r\nPrint the Page:\r\n");

	HDC hdcPrint;               // printer DC handle 
    char szDevString[120];      // array for WIN.INI data 
    char *szPrinter, *szDriver; // printer and driver names 
    char *szPort;               // port name 
	
    //Retrieve the printer, printer driver, and output-port names from WIN.INI. 
    GetProfileStringA("windows", "device", ",,,", 
        szDevString, 120); 
	
    //Parse the string of names, setting ptrs as required 
    // If the string contains the required names, use them to 
    // create a device context. 
    if ((szPrinter = strtok(szDevString, 
		(const char *) ",")) 
		&& (szDriver = strtok ((char *) NULL, 
		(const char *) ", ")) 
		&& (szPort = strtok ((char *) NULL, 
		(const char *) ", "))) 
    {
        hdcPrint = CreateDCA(szDriver, szPrinter, 
            szPort, NULL); 
    }
	
	if (!hdcPrint)
	{
		//Fails when create printer DC handle.
		strLog += "Fails when create printer DC handle\r\n";
		//Output the log string.
		FSDK_OutputStringLog(strLog.c_str(), (int)strLog.length());
		return FSCRT_ERRCODE_ERROR;
	}

	//Zero and then initialize the members of a DOCINFO structure.
	FS_INT32 nPageIndex = 0;
	FS_RESULT ret = FSPDF_Page_GetIndex(page, &nPageIndex);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		//Fails when create printer DC handle.
		strLog += "Fail to get page index.\r\n";
		//Output the log string.
		FSDK_OutputStringLog(strLog.c_str(), (int)strLog.length());
		return FSCRT_ERRCODE_ERROR;
	}
	char sDocName[128] = "";
	sprintf(sDocName, "FoxitPDFSDK_PrintingTest_Page%d", nPageIndex+1);
	DOCINFOA docInfo;
	ZeroMemory(&docInfo, sizeof(DOCINFOA));
	docInfo.cbSize = sizeof(DOCINFOA);
	docInfo.lpszDocName = sDocName; 
	docInfo.lpszOutput = NULL; 
	docInfo.fwType = 0; 
	//Begin a print job by calling the StartDoc function.
	StartDocA(hdcPrint, &docInfo);
	//Inform the driver that the application is about to begin sending data.
	StartPage(hdcPrint);

	//Render the page by newer device context.
	FS_FLOAT nPageWidth = (FS_FLOAT)GetDeviceCaps(hdcPrint, HORZRES);
	FS_FLOAT nPageHeight = (FS_FLOAT)GetDeviceCaps(hdcPrint, VERTRES);
	FS_FLOAT fXScale = nPageWidth/fPageWidth;
	FS_FLOAT fYScale = nPageHeight/fPageHeight;

	FS_FLOAT fScale = fXScale < fYScale ? fXScale : fYScale;
	FS_FLOAT fOffsetX = 0.0;
	FS_FLOAT fOffsetY = 0.0;
	if( fXScale < fYScale)
	{
		fOffsetY = (nPageHeight - fScale * fPageHeight)/2;
		nPageHeight = fScale * fPageHeight;
	}
	else
	{
		fOffsetX = (nPageWidth - fScale * fPageWidth)/2;
		nPageWidth = fScale * fPageWidth;
	}

	RECT rect = {(LONG)fOffsetX, (LONG)fOffsetY, (LONG)(nPageWidth + fOffsetX), (LONG)(nPageHeight + fOffsetY)};
	renderPage(hdcPrint, rect, page);
	
	//Print the page success.
	strLog += "success\r\n";
	//Output the log string.
	FSDK_OutputStringLog(strLog.c_str(), (int)strLog.length());
	//Inform the spooler that the page is complete. 
	EndPage(hdcPrint);
	//Inform the driver that document has ended. 
	EndDoc(hdcPrint);
	//Release the device context.
	DeleteDC(hdcPrint);

	return FSCRT_ERRCODE_SUCCESS;
}

/*******************************************************************************/
/*  Print the PDF document procedure.						                   */
/*******************************************************************************/
FS_RESULT PrintProc(string fileName, string strFile)
{
	//Define a string object to store log data.
	string strLog("\r\n======");
	//Get PDF file name from parameter, and output to log.
//	string strFile = FSDK_GetInputFilesFolder("pdfprint");
	strFile += "/";
	strFile += fileName;
	strLog += fileName;

	//Open a PDF file.
	strLog += "\r\n\topen file: ";
	FSCRT_FILE file = FSDK_OpenFile(strFile.c_str(), "rb");
	if (!file)
	{
		//Failure, output log string and return.
		strLog += "failure\r\n";
		FSDK_OutputStringLog(strLog.c_str(), (int)strLog.length());
		return FSCRT_ERRCODE_ERROR;
	}
	//Open successfully.
	strLog += "success\r\n";

	//Load the PDF document.
	strLog += "\tload PDF document: ";
	FSCRT_DOCUMENT pdfDoc = NULL;
	FS_RESULT ret = FSPDF_Doc_StartLoad(file, NULL, &pdfDoc, NULL);
	if (ret == FSCRT_ERRCODE_SUCCESS)
	{
		//Load document successfully.
		strLog += "success\r\n";

		//Load a PDF page by index.
		FS_INT32 nPageCount = 0;
		FS_INT32 nPageNum = 0;
		FS_INT32* nPageIndexArray = NULL;
		if (g_PageRange != "")
		{
			if (FSDK_GetPageRange(g_PageRange.c_str(), nPageIndexArray, &nPageNum))
			{
				//Count page number
				ret = FSPDF_Doc_CountPages(pdfDoc, &nPageCount);
				if (ret != FSCRT_ERRCODE_SUCCESS)
				{
					strLog += "Failed to get page count\r\n";
					FSDK_OutputStringLog(strLog.c_str(), (int)strLog.length());
					if (nPageIndexArray) 
						delete[] nPageIndexArray;
					nPageIndexArray = NULL;
					if (pdfDoc) 
						FSPDF_Doc_Close(pdfDoc);
					return ret;
				}
			}
		}
		if (nPageNum < 1) nPageNum = 1;

		//Dealing with page one by one.
		for (int i=0; i<nPageNum; i++)
		{
			FS_INT32 pageIndex;
			if (g_PageRange != "" && nPageIndexArray)
			{
				if (nPageIndexArray[i] >= nPageCount || nPageIndexArray[i] < 0)
				{
					FSDK_OutputLog("failure: Page index %d is out of valid range!\r\n", nPageIndexArray[i]);
					continue;
				}
				pageIndex = nPageIndexArray[i];
			}
			else
			{
				pageIndex = i;
			}
			FSCRT_PAGE pdfPage = NULL;
			char strTemp[128] = "";
			sprintf(strTemp, "========Page Index %d ========\r\nLoading: ", pageIndex);
			strLog += strTemp;
			ret = FSPDF_Doc_GetPage(pdfDoc, pageIndex, &pdfPage);
			if (ret == FSCRT_ERRCODE_SUCCESS)
			{
				//Load page successfully.
				strLog += "success\r\n";

				//Start parsing PDF page.
				FSCRT_PROGRESS progress = NULL;
				FS_RESULT ret = FSPDF_Page_StartParse(pdfPage, FSPDF_PAGEPARSEFLAG_NORMAL, &progress);
				if (ret != FSCRT_ERRCODE_FINISHED) {
					//Pause the parse process when unknown error occurs.
					if (ret != FSCRT_ERRCODE_SUCCESS) {
						//Release all page content objects.
						FSPDF_Page_Clear(pdfPage);
						return FALSE;
					}

					//Continue a progressive process.
					ret = FSCRT_ERRCODE_TOBECONTINUED;
					while (ret == FSCRT_ERRCODE_TOBECONTINUED || ret == FSCRT_ERRCODE_ROLLBACK) {
						ret = FSCRT_Progress_Continue(progress, NULL);
					}
					//Release the current progressive process.
					FSCRT_Progress_Release(progress);
				}

				//Print the current page.
				printPage(pdfPage);

				//Release all page content objects.
				FSPDF_Page_Clear(pdfPage);
				pdfPage = NULL;
			}
			else
				//Fails when load PDF page.
				strLog += "failure\r\n";
		}
		if (nPageIndexArray)
		{
			delete []nPageIndexArray;
			nPageIndexArray = NULL;
		}
		//Close PDF document.
		FSPDF_Doc_Close(pdfDoc);
		pdfDoc = NULL;
	}
	else
		//Fails when load PDF file.
		strLog += "failure\r\n";

	//Release file object.
	FSCRT_File_Release(file);
	//Output total log string.
	FSDK_OutputStringLog(strLog.c_str(), (int)strLog.length());

	return ret;
}
//print pdf file to output folder.
void PirntPDFFile(string pdfFile, string inputFilesPath, string outputFilesPath)
{
	if (pdfFile.size() > 0 && FSDK_CompareExtName(pdfFile.c_str(), ".pdf"))
	{

		//Load PDF Document and print will be done if success.
		FSDK_OutputLog("Load PDF Document and print:\r\n");
		if (FSCRT_ERRCODE_SUCCESS == PrintProc(pdfFile, inputFilesPath))
			FSDK_OutputLog("success\r\n");
		else
			FSDK_OutputLog("failure\r\n");

	}
}
//print all pdf file in inputFilesPath folder.
void PrintPDFFolder(void* folder, string inputFilesPath, string outputFilesPath)
{
	do
	{
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		if(!isDir)
		{
			PirntPDFFile(strFile, inputFilesPath, outputFilesPath);
		}
		if (strFile.size() < 1) break;

	} while (1);
	//Close a folder.
	FSDK_CloseFolder(folder);
	FSDK_OutputLog("success\r\n");
}

/*******************************************************************************/
/*  Defines the entry point for the console application.                       */
/*******************************************************************************/
int main(int argc, char* argv[])
{
	//Set the current executing file path.
	FSDK_SetExecuteFilePath(argc, argv);
	string inputSrcFile = "";
	string outputFolder = "";
	//Judge if to command line arguments. 
	//usage: pdfprint <srcfile> /r [pagerange] /o [destfolder]
	if (argc > 1)
	{
		inputSrcFile = argv[1];
		bool bArgumentValid = true;
		switch(argc)
		{
		case 2:
			break;
		case 3:
			{
				if (0 != strcmp(argv[2], "/r") && 0 != strcmp(argv[2], "/o"))
					bArgumentValid = false;
				break;
			}
		case 4:
			{//usage: pdfprint <srcfile> XXX XXX
				if (0 != strcmp(argv[2], "/r") && 0 != strcmp(argv[2], "/o"))
					bArgumentValid = false;
				else if (0 == strcmp(argv[2], "/r"))
				{
					if (0 != strcmp(argv[3], "/o") && '/'!=argv[3][0])
						g_PageRange = argv[3]; //usage: pdfprint <srcfile> /r [pagerange]
					else if (0 != strcmp(argv[3], "/o") && '/'==argv[3][0])
						bArgumentValid = false;
				}
				else if (0 == strcmp(argv[2], "/o"))
					outputFolder = argv[3]; //usage: pdfprint <srcfile> /o [destfolder]
				break;
			}
		case 5:
			{//usage: pdfprint <srcfile> /r XXX XXX
				if (0 != strcmp(argv[2], "/r"))
					bArgumentValid = false;
				else 
				{
					if (0 != strcmp(argv[3], "/o"))
					{
						if (0 != strcmp(argv[4], "/o"))
							bArgumentValid = false;
						else
							g_PageRange = argv[3]; //usage: pdfprint <srcfile> /r [pagerange] /o
					}
					else
						outputFolder = argv[4];//usage: pdfprint <srcfile> /r /o [destfolder]
				}
				break;
			}
		case 6:
			{//usage: pdfprint <srcfile> /r [pagerange] /o [destfolder]
				if (0 != strcmp(argv[2], "/r") || 0 != strcmp(argv[4], "/o"))
					bArgumentValid = false;
				else
				{
					g_PageRange = argv[3];
					outputFolder = argv[5];
				}
				break;
			}
		default:
			bArgumentValid = false;
		}
		if (false == bArgumentValid || !FSDK_FileExist(inputSrcFile.c_str()) || (outputFolder != "" && !FSDK_FolderExist(outputFolder.c_str())))
		{
			printf("\r\n");
			printf("usage: pdfprint [<srcfile> /r [pagerange] /o [destfolder]] \r\n\r\n");
			printf("<srcfile>\t: Path of a PDF file as input file.\r\n");
			printf("/r [pagerange]\t: Page range, used to specify indexes of PDF pages in input PDF\r\n ");
			printf("\t\t  file, such as \"0\", \"1-3,5\". Page index starts from 0 and\r\n\t\t  should be valid within page count. Invalid index will be\r\n\t\t  ignored. "\
				"If input page range is totally invalid, page range is\r\t\t  to be first page by default.\r\n");
			printf("/o [destfolder]\t: Path of a folder as output folder. If not set, use default\r\n\t\t  output folder.\r\n\r\n");
			return -1;
		}
	}
	// Locate the path of log file
	string logPath ;
	if (outputFolder != "")
	{
		outputFolder += "/pdfprint";
		FSDK_CreatFolder(outputFolder.c_str());
		logPath = outputFolder;
		logPath += "/log.txt";
	}
	else
	{
		outputFolder = FSDK_GetOutputFilesFolder("pdfprint");
		logPath = FSDK_GetLogFile("pdfprint");
	}
	
	//Open and write log
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdfprint\r\n");
	FSDK_OutputLog("\tInput from: input_files folder\r\n");
	FSDK_OutputLog("\tOutput to: output_files/pdfprint\r\n\r\n");

	//Initialize SDK library, or no method of SDK can be used.
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		FSDK_OutputLog("failure\r\n");
		FSDK_CloseLog();
		return -1;
	}
	FSDK_OutputLog("success\r\n");

	//Initialize PDF module, or no method of PDF module can be used.
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(outputFolder + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	//Enumerate files in specific folder and choose PDF file to load.
	string inputFolder;
	string pdfName;
	if (inputSrcFile != "")
	{
		pdfName = FSDK_GetFileName(inputSrcFile.c_str());
		inputFolder = FSDK_GetCustomInputPath(inputSrcFile.c_str());
		PirntPDFFile(pdfName, inputFolder, "");
	}
	else
	{
		inputFolder = FSDK_GetInputFilesFolder("pdfprint");
		//Open a folder to enumerate all files in it.
		void* folder = FSDK_OpenFolder(inputFolder.c_str());
		if (folder)
		{
			FSDK_OutputLog("\r\nEnumerate files in folder input_files/pdfprint: ");
			PrintPDFFolder(folder, inputFolder, "");
		}
		else
		{
			pdfName = "AboutFoxit.pdf";
			inputFolder = FSDK_GetFixFolder();
			PirntPDFFile(pdfName, inputFolder, "");
		}
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);
	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}