﻿#include "StdAfx.h"
#include "PDFReaderPage.h"
#include "PDFReaderDocument.h"
#include "AnnotHandlerMgr.h"

const extern char* g_AnnotTypeList[] = 
{
	FSPDF_ANNOTTYPE_TEXT,
	FSPDF_ANNOTTYPE_LINK,
	FSPDF_ANNOTTYPE_FREETEXT,
	FSPDF_ANNOTTYPE_LINE,
	FSPDF_ANNOTTYPE_SQUARE,
	FSPDF_ANNOTTYPE_CIRCLE,
	FSPDF_ANNOTTYPE_POLYGON,
	FSPDF_ANNOTTYPE_POLYLINE,
	FSPDF_ANNOTTYPE_HIGHLIGHT,
	FSPDF_ANNOTTYPE_UNDERLINE,
	FSPDF_ANNOTTYPE_SQUIGGLY,
	FSPDF_ANNOTTYPE_STRIKEOUT,
	FSPDF_ANNOTTYPE_STAMP,
	FSPDF_ANNOTTYPE_CARET,
	FSPDF_ANNOTTYPE_INK,
	FSPDF_ANNOTTYPE_POPUP,
	FSPDF_ANNOTTYPE_FILEATTACHMENT,
	FSPDF_ANNOTTYPE_SOUND,
	FSPDF_ANNOTTYPE_MOVIE,
	FSPDF_ANNOTTYPE_WIDGET,
	FSPDF_ANNOTTYPE_SCREEN,
	FSPDF_ANNOTTYPE_PRINTERMARK,
	FSPDF_ANNOTTYPE_TRAPNET,
	FSPDF_ANNOTTYPE_WATERMARK,
	FSPDF_ANNOTTYPE_PSI,
	FSPDF_ANNOTTYPE_3D
};

CReader_Page::CReader_Page(CReader_Document* pReaderDocument)
	: m_nIndex(0)
	, m_fHeight(0)
	, m_fWidth(0)
	, m_fTotalHeight(0)
	, m_fPageRealWidth(0)
	, m_bIsParserTextLink(FALSE)
	, m_pReaderDocument(pReaderDocument)
	, m_TextLink(NULL)
	, m_TextPage(NULL)
	, m_TextSearch(NULL)
	, m_bIsLoadAnnot(FALSE)
	, m_bAnnotFocus(FALSE)
	, m_bIsParserPage(FALSE)
	, m_pdfPage(NULL)
	, m_bHasTransparency(FALSE)
{
}

CReader_Page::~CReader_Page()
{
	int nSize = m_pAnnotList.GetSize();
	for (int i = nSize-1; i>= 0; i--)
	{
		IReader_Annot* pReaderAnnot = m_pAnnotList.GetAt(i);
		delete pReaderAnnot;
	}
	m_pAnnotList.RemoveAll();
	ReleaseTempData();
}

IReader_Document* CReader_Page::GetDocument()
{
	return (IReader_Document*)m_pReaderDocument;
}

FSCRT_PAGE CReader_Page::GetPage()
{
	return m_pdfPage;
}

FSPDF_TEXTLINK CReader_Page::GetTextLink()
{
	return m_TextLink;
}

BOOL CReader_Page::IsParseTextLink()
{
	return m_bIsParserTextLink;
}

BOOL CReader_Page::ParsePage()
{
	if (m_bIsParserPage) return TRUE;

	//Before loading a text page from a PDF page, the PDF page should be parsed at first.

	FSCRT_PAGE pdfPage = NULL;
	FS_RESULT ret = FSPDF_Doc_GetPage(m_pReaderDocument->GetDocument(), m_nIndex, &pdfPage);
	 
	if ( FSCRT_ERRCODE_SUCCESS != ret) return NULL;
	 
	m_pdfPage = pdfPage;
	if (!m_pdfPage) return FALSE;
	

	FSCRT_PROGRESS progress = NULL;
	
	ret = FSPDF_Page_StartParse(m_pdfPage, FSPDF_PAGEPARSEFLAG_NORMAL, &progress);
	if (ret == FSCRT_ERRCODE_FINISHED)
	{
		return TRUE;
	}
	if (FSCRT_ERRCODE_SUCCESS != ret || progress == NULL)
	{
		return FALSE;
	}

	//Continue to parse
	ret = FSCRT_Progress_Continue(progress, NULL);
	if (FSCRT_ERRCODE_FINISHED != ret)
	{
		return FALSE;
	}
	
	//Release progress resource
	FSCRT_Progress_Release(progress);

	FSPDF_Page_DoJavaScriptAction(pdfPage, FSPDF_TRIGGER_PAGE_PO);
	FSPDF_Page_DoJavaScriptAction(pdfPage, FSPDF_TRIGGER_PAGE_PC);

	m_bIsParserPage = TRUE;

	FSPDF_Page_HasTransparency(pdfPage, &m_bHasTransparency);

	return TRUE;
}

int CReader_Page::GetPageIndex()
{
	return m_nIndex;
}

BOOL CReader_Page::DocToWindow(FSCRT_RECTF rcDoc, CRect& rcWindow)
{
	return m_pReaderDocument->DocToWindow(m_nIndex, rcDoc, rcWindow);
}

BOOL CReader_Page::WindowToDoc(int window_x, int window_y, FS_FLOAT& doc_x, FS_FLOAT& doc_y)
{
	return m_pReaderDocument->WindowToDoc(m_nIndex, window_x, window_y, doc_x, doc_y);
}

BOOL CReader_Page::WindowToDoc(CRect rcWindow, FSCRT_RECTF& rcDoc)
{
	return m_pReaderDocument->WindowToDoc(m_nIndex, rcWindow, rcDoc);
}

BOOL CReader_Page::WindowToDocForPrint(int nPageIndex, CRect rcWindow, FSCRT_RECTF& rcDoc)
{
	return m_pReaderDocument->WindowToDocForPrint(nPageIndex, rcWindow, rcDoc);
}

IReader_Annot* CReader_Page::AddAnnot(LPCSTR lpSubType, FSCRT_RECTF rect)
{
	FSCRT_BSTR annotType;
	annotType.str = (FS_LPSTR)lpSubType;
	annotType.len = strlen(lpSubType);

	if (!m_pdfPage) return NULL;

	int nSize = m_pAnnotList.GetSize();
	FSCRT_ANNOT pdfAnnot = NULL;
	FS_RESULT hResult = FSPDF_Annot_Add(m_pdfPage, &rect, &annotType, &annotType, nSize + 1, &pdfAnnot);
	if (hResult != FSCRT_ERRCODE_SUCCESS) return NULL;

	GetDocument()->SetChangeMark();
	//Set annotation common properties.
	setAnnotProperties(pdfAnnot, annotType.str);
	//Set Highlight annotation properties.
	setHighlightAnnotProperties(pdfAnnot, annotType.str, rect);
	//Set UnderLine annotation properties.
	setUnderLineAnnotProperties(pdfAnnot, annotType.str, rect);
	//Set FreeText annotation properties.
	setFreeTextAnnotProperties(pdfAnnot, annotType.str, rect);
	//Set Link annotation properties.
	setLinkAnnotProperties(pdfAnnot, annotType.str);
	//Set Text annotation properties. 
	setTextAnnotProperties( pdfAnnot, annotType.str);

	IReader_App* pReaderApp = m_pReaderDocument->GetApp();
	CReader_AnnotHandlerMgr* pAnnotHandlerMgr = (CReader_AnnotHandlerMgr*)pReaderApp->GetAnnotHandlerMgr();
	IReader_Annot* pReaderAnnot = NULL;
	if(pAnnotHandlerMgr)
	{
		pReaderAnnot = pAnnotHandlerMgr->NewAnnot(pdfAnnot, this);
	}
	if(!pReaderAnnot) return NULL;
	m_pAnnotList.Add(pReaderAnnot);	
	return pReaderAnnot;
}

BOOL CReader_Page::DeleteAnnot(IReader_Annot* pAnnot)
{
	INT_PTR nCount = m_pAnnotList.GetCount();
	for (INT_PTR i = 0; i < nCount; i++)
	{
		IReader_Annot* pGetAnnot = m_pAnnotList.GetAt(i);
		if (pGetAnnot == pAnnot)
		{
			FSPDF_Annot_Remove(pAnnot->GetPDFAnnot());
			m_pAnnotList.RemoveAt(i);
			GetDocument()->SetCapturedAnnot(NULL);
			GetDocument()->SetSelectedAnnot(NULL);
			GetDocument()->SetChangeMark();
			delete pGetAnnot;
			pGetAnnot = NULL;
			return TRUE;
		}
	}
	return FALSE;
}

void CReader_Page::MoveAnnot(IReader_Page* pDestPage, IReader_Annot *pAnnot)
{
}

int CReader_Page::CountAnnots()
{
	return m_pAnnotList.GetSize();
}

IReader_Annot* CReader_Page::GetAnnot(int index)
{
	return m_pAnnotList.GetAt(index);
}

// there can be only one focus annotation in the page, and actually in one document
IReader_Annot* CReader_Page::GetFocusAnnot()
{
	return NULL;
}

void CReader_Page::SetFocusAnnot(IReader_Annot* pAnnot)
{
}

void CReader_Page::SetFocusAnnotFlag(BOOL bFlag)
{
	m_bAnnotFocus = bFlag;
}

void CReader_Page::SetPageIndex(int nIndex)
{
	m_nIndex = nIndex;
}
void CReader_Page::SetPageHeight(float fHeight)
{
	m_fHeight = fHeight;
}
void CReader_Page::SetPageWidth(float fWidth)
{
	m_fWidth = fWidth;
}
void CReader_Page::SetPageTotalHeight(float fTotalHeight)
{
	m_fTotalHeight = fTotalHeight;
}
void CReader_Page::SetPageRealWidth(float fPageRealWidth)
{
	m_fPageRealWidth = fPageRealWidth;
}

void CReader_Page::SetPDFPageTextLink(FSPDF_TEXTLINK textLink)
{
	m_TextLink = textLink;
	m_bIsParserTextLink = TRUE;
}

void CReader_Page::SetPDFTextPage(FSPDF_TEXTPAGE textPage)
{
	m_TextPage = textPage;
}

FSPDF_TEXTPAGE CReader_Page::GetPDFTextPage()
{
	 return m_TextPage;
}

FSPDF_TEXTSEARCH CReader_Page::GetPDFTextSearch()
{
	return m_TextSearch;
}

FS_BOOL CReader_Page::LoadPDFTextPage()
{
	if (!m_pdfPage) return FALSE;
	
	FS_RESULT ret = FSPDF_TextPage_Load(m_pdfPage, &m_TextPage);
	if (FSCRT_ERRCODE_INVALIDLICENSE == ret && m_pReaderDocument->m_bLoadText)
	{
		AfxMessageBox(_T("You don't have the right of \"FSPDF_TextPage_Load\" Functions, please purchase \"Edit Add-on\" Modules!"));
		m_pReaderDocument->m_bLoadText = FALSE;
	}
	if (FSCRT_ERRCODE_SUCCESS == ret)
	{
		return TRUE;
	}
	return FALSE;
}

void CReader_Page::ReleasePDFTextPage()
{
	if (m_TextSearch)
	{
		FSPDF_TextSearch_Release(m_TextSearch);
		m_TextSearch = NULL;
	}

	if (m_TextPage)
	{
		FSPDF_TextPage_Release(m_TextPage);
		m_TextPage = NULL;
	}
}

FS_BOOL CReader_Page::StartSearch(const FSCRT_BSTR* searchPattern, FS_DWORD flags, FS_INT32 startIndex)
{
	if (m_TextPage)
	{
		FS_RESULT ret = FSPDF_TextPage_StartSearch( m_TextPage, searchPattern, flags, startIndex, &m_TextSearch);
		if( FSCRT_ERRCODE_SUCCESS == ret)
		{
			return TRUE;
		}
	}
	return FALSE;
}

FS_BOOL CReader_Page::FindNext()
{
	FS_BOOL isMatch = FALSE;
	FSPDF_TextSearch_FindNext(m_TextSearch, &isMatch);
	return isMatch;
}

FS_BOOL CReader_Page::FindPrev()
{
	FS_BOOL isMatch = FALSE;
	FSPDF_TextSearch_FindPrev(m_TextSearch, &isMatch);
	return isMatch;
}

FS_BOOL CReader_Page::ContinueSearch(FS_INT32 nDirection, const FSCRT_BSTR* searchPattern, FS_DWORD flags, FS_INT32 startIndex, FS_INT32* pFindPageIndex)
{
	FS_BOOL isMatch = FALSE;
	int pageCount = 0;
	FS_RESULT ret = FSPDF_Doc_CountPages(m_pReaderDocument->GetDocument(), &pageCount);
	int nPageIndex = 0;
	nPageIndex = GetPageIndex();
	while (!isMatch)
	{
		if (0 == nDirection)
		{
			nPageIndex++;
		}
		else
		{
			nPageIndex--;
		}

		if (nPageIndex>= pageCount || nPageIndex<0)
		{
			break;
		}
		
		if (NULL == m_pReaderDocument)
		{
			break;
		}

		CReader_Page* pPage = (CReader_Page*)m_pReaderDocument->GetPage(nPageIndex);
		if (NULL == pPage)
		{
			break;
		}
	
		pPage->ParsePage();

		if (!pPage->GetPDFTextPage())
		{
			pPage->LoadPDFTextPage();
		}
		
		pPage->StartSearch(searchPattern, flags, startIndex);
		if (0 == nDirection)// find down
		{
			isMatch = pPage->FindNext();
		}
		else
		{
			isMatch = pPage->FindPrev();
		}

		if (!isMatch)
		{
			pPage->ReleasePDFTextPage();
		}
		else
		{
			*pFindPageIndex = pPage->GetPageIndex();
		}
	}
	return isMatch;
}

BOOL CReader_Page::LoadAnnots()
{
	if(m_bIsLoadAnnot) 
		return TRUE;
	//FSCRT_PAGE pdfPage = GetPage();
	if (!m_pdfPage) return FALSE;
	
	FS_RESULT ret = FSPDF_Page_LoadAnnots(m_pdfPage);
	if (FSCRT_ERRCODE_INVALIDLICENSE == ret && m_pReaderDocument->m_bLoadAnnot)
	{
		AfxMessageBox(_T("You don't have the right of \"FSPDF_Page_LoadAnnots\" Functions, please purchase \"Annot Add-on\" Modules!"));
		m_pReaderDocument->m_bLoadAnnot = FALSE;
	}
	if (ret != FSCRT_ERRCODE_SUCCESS) return FALSE;
	
	IReader_App* pReaderApp = m_pReaderDocument->GetApp();
	CReader_AnnotHandlerMgr* pAnnotHandlerMgr = (CReader_AnnotHandlerMgr*)pReaderApp->GetAnnotHandlerMgr();

	int nAnnotTypeCount = sizeof(g_AnnotTypeList)/sizeof(char*);
	for(int i = 0; i<nAnnotTypeCount; i++)
	{
		int nCount = 0;
		FSCRT_BSTR filter = {(FS_LPSTR)g_AnnotTypeList[i], strlen(g_AnnotTypeList[i])};
		ret = FSPDF_Annot_GetCount(m_pdfPage, &filter, &nCount);
		for(int j = 0; j<nCount; j++)
		{
			FSCRT_ANNOT pdfAnnot = NULL;
			FSPDF_Annot_Get(m_pdfPage, &filter, j, &pdfAnnot);
			if (!pdfAnnot) continue;

			IReader_Annot* pReaderAnnot = NULL;
			if(pAnnotHandlerMgr)
			{
				pReaderAnnot = pAnnotHandlerMgr->NewAnnot(pdfAnnot, this);
			}
			if(!pReaderAnnot) continue;

			m_pAnnotList.Add(pReaderAnnot);
		}
	}
	m_bIsLoadAnnot = TRUE;
	return TRUE;;
}

IReader_Annot* CReader_Page::GetAnnotAtPoint(CPoint point, CString csType)
{
	IReader_App* pReaderApp = m_pReaderDocument->GetApp();
	CReader_AnnotHandlerMgr* pAnnotHandlerMgr = (CReader_AnnotHandlerMgr*)pReaderApp->GetAnnotHandlerMgr();
	if(!pAnnotHandlerMgr) return NULL;
	int nSize = m_pAnnotList.GetSize();
	for (int i = 0; i<nSize; i++)
	{
		IReader_Annot* pReaderAnnot = m_pAnnotList.GetAt(i);
		CRect rcWindow = pAnnotHandlerMgr->Annot_OnGetViewBBox(this, pReaderAnnot);
		CString csSubType;

		if (FSCRT_ANNOT pdfAnnot = pReaderAnnot->GetPDFAnnot())
		{
			FSCRT_BSTR annotType = {NULL, 0};
			FSPDF_Annot_GetType(pdfAnnot, &annotType);

			wchar_t* buf = NULL;
			FS_DWORD length = 0;
			FS_RESULT hResult = FSCRT_UTF8_ToUTF16LE(&annotType, NULL, &length);
			if(hResult == FSCRT_ERRCODE_SUCCESS)
			{
				buf = new wchar_t[length+1];
				memset(buf, 0, (length+1)*sizeof(wchar_t));
				FSCRT_UTF8_ToUTF16LE(&annotType, (FS_WORD*)buf, &length);
				csSubType = buf;
				delete [] buf;
			}
		}

		if( rcWindow.PtInRect(point) && (csType.IsEmpty() || (csSubType == csType)))
		{
			//[do further judgment by AnnotHandler]
			if(pAnnotHandlerMgr->Annot_OnHitTest(this, pReaderAnnot, point))
			{
				return pReaderAnnot;
			}
		}
	}
	return NULL;
}

float CReader_Page::GetPageTotalHeight()
{
	return m_fTotalHeight;
}
float CReader_Page::GetPageRealWidth()
{
	return m_fPageRealWidth;
}

float CReader_Page::GetPageHeight()
{
	return m_fHeight;
}
float CReader_Page::GetPageWidth()
{
	return m_fWidth;
}

BOOL CReader_Page::IsHasTransparency()
{
	return m_bHasTransparency;
}

void CReader_Page::ReleaseTempData()
{
	//page have focus annot return
	if (m_bAnnotFocus) 
	{
		return;
	}

	if (m_TextLink)
	{
		FSPDF_TextLink_Release(m_TextLink);
		m_TextLink = NULL;
		m_bIsParserTextLink = FALSE;
	}

	if (m_TextSearch)
	{
		FSPDF_TextSearch_Release(m_TextSearch);
		m_TextSearch = NULL;
	}

	if (m_TextPage)
	{
		FSPDF_TextPage_Release(m_TextPage);
		m_TextPage = NULL;
	}

	if (m_pdfPage)// 
	{
		 FSPDF_Page_UnloadAnnots(m_pdfPage);
		 m_bIsLoadAnnot = FALSE;
		 int nSize = m_pAnnotList.GetSize();
		 for (int i = nSize-1; i>= 0; i--)
		 {
			 IReader_Annot* pReaderAnnot = m_pAnnotList.GetAt(i);
			 delete pReaderAnnot;
		 }
		 m_pAnnotList.RemoveAll();
		 FSPDF_Page_Clear(m_pdfPage);
		 m_bIsParserPage = FALSE;
		 m_pdfPage = NULL;
	}
}

//Set annotation common properties.
void CReader_Page::setAnnotProperties(FSCRT_ANNOT annot, char* sAnnotType)
{
	//Prepare the string object of the annotation name.
	FSCRT_BSTR bsAnnotName;
	FSCRT_BStr_Init(&bsAnnotName);
	FSCRT_BStr_Set(&bsAnnotName, sAnnotType, (FS_DWORD)strlen(sAnnotType));

	//Set the flags, name of annotation.
	FSPDF_Annot_SetFlags(annot, FSPDF_ANNOTFLAG_NOROTATE | FSPDF_ANNOTFLAG_PRINT);
	FSPDF_Annot_SetName(annot, &bsAnnotName);

	setAnnotBorder(annot);

	//Set the subject, title, contents, icon name of annotation.
	FSPDF_Annot_SetSubject(annot, &bsAnnotName);
	FSPDF_Annot_SetTitle(annot, &bsAnnotName);
	unsigned short* utf16le = (unsigned short*)L"FreeText";
	//FSCRT_BSTRC(content, "FreeTextTestinglong\nTexttestTest测试اختبار測試テストprovë테스트tèsפּרובירδοκιμή");
	FSCRT_BSTR content;
	FSCRT_BStr_Init(&content);

#if defined(_WIN32) || defined(_WIN64)
	FSCRT_UTF8_FromUTF16LE(&content, (const FS_WORD*)utf16le, wcslen((wchar_t*)utf16le));
#else//temp for linux
	FSCRT_UTF8_FromUTF32LE(&content, (const FS_DWORD*)utf16le, wcslen((wchar_t*)utf16le));
#endif

	FSPDF_Annot_SetContents(annot, &content);
	//FSPDF_Annot_SetIconName(annot, &bsAnnotName);
	FSCRT_BStr_Clear(&content);
	//Set the stroke color and the opacity of annotation.
	FSPDF_Annot_SetColor(annot, FALSE, 0x00FF0000);
	FSPDF_Annot_SetOpacity(annot, 1.0);
	// 	FSCRT_BSTRC(intent, FSPDF_ANNOT_INTENTNAME_FREETEXT_TYPEWRITER);
	// 	FSPDF_Annot_SetIntent(annot, &intent);

	//Set the creation time and the modification time of annotation.
	FSCRT_DATETIMEZONE dt;
	dt.day = 14;
	dt.month = 1;
	dt.year = 2014;
	FSPDF_Annot_SetCreationDateTime(annot, &dt);
	FSPDF_Annot_SetModifiedDateTime(annot, &dt);

	//Clear the string object.
	FSCRT_BStr_Clear(&bsAnnotName);
}

//Set Highlight annotation properties.
void CReader_Page::setHighlightAnnotProperties(FSCRT_ANNOT annot, char* sAnnotType, FSCRT_RECTF rect)
{
	//Deal with the Highlight annotation only.
	if(strcmp(sAnnotType, FSPDF_ANNOTTYPE_HIGHLIGHT)) return;

	//Set the quadrilaterals points of annotation.
	FSCRT_QUADPOINTSF quadPoints = {rect.left, rect.top, rect.right, rect.top, rect.left, rect.bottom, rect.right, rect.bottom};//Reference:{100, 110, 300, 110, 300, 300, 100, 300};
	FSPDF_Annot_SetQuadPoints(annot, &quadPoints, 1);
	//Set the stroke color and opacity of annotation.
	FSPDF_Annot_SetColor(annot, FALSE, 0x0000FFf0);
	FSPDF_Annot_SetOpacity(annot, (FS_FLOAT)0.55);

	//Add the specified state annotation to an annotation.
	FSCRT_ANNOT stateAnnot = NULL;
	FS_RESULT ret = FSPDF_Annot_AddState(annot, FSPDF_ANNOTSTATEMODEL_REVIEW, FSPDF_ANNOTSTATE_REVIEW_ACCEPTED, &stateAnnot);
	if (FSCRT_ERRCODE_SUCCESS == ret)
		//Remove the state annotation.
		FSPDF_Annot_Remove(stateAnnot);
}

//Set Text annotation properties.
void CReader_Page::setTextAnnotProperties(FSCRT_ANNOT annot, char* sAnnotType)
{

	//Deal with the Highlight annotation only.
	if(strcmp(sAnnotType, FSPDF_ANNOTTYPE_TEXT)) return;

	FSCRT_BSTR NoteType;
	FSCRT_BStr_Init(&NoteType);
	FSCRT_BStr_Set(&NoteType, "Comment", sizeof("Comment"));
	FSPDF_Annot_SetIconName(annot, &NoteType);
	FSCRT_BStr_Clear(&NoteType);

}

//Set UnderLine annotation properties.
void CReader_Page::setUnderLineAnnotProperties(FSCRT_ANNOT annot, char* sAnnotType, FSCRT_RECTF rect)
{
	//Deal with the UnderLine annotation only.
	if(strcmp(sAnnotType, FSPDF_ANNOTTYPE_UNDERLINE)) return;

	FSPDF_TEXTPAGE hTextPage = NULL;
	FS_RESULT ret = FSPDF_TextPage_Load(m_pdfPage, &hTextPage);

	FSPDF_TEXTSELECTION textSelection = NULL;
	ret = FSPDF_TextPage_SelectByRectangle(hTextPage, &rect, &textSelection);

	FS_INT32 pieceCount;
	ret = FSPDF_TextSelection_CountPieces(textSelection, &pieceCount);
	if (pieceCount > 0)
	{
		FSCRT_QUADPOINTSF* quadPoints = new FSCRT_QUADPOINTSF[pieceCount];
		for (int i = 0; i < pieceCount; i++)
		{
			FSCRT_RECTF rectOfPiece;
			FSPDF_TextSelection_GetPieceRect(textSelection, i, &rectOfPiece);
			FS_INT32 rotation = 0;
			ret =FSPDF_TextSelection_GetPieceRotation(textSelection,i,&rotation);
			//To fix SDK-5182, refer to unittest TEST_F(FSPDF_Annot_Test, Test_SDK4581_UnderlineAnnotEffect).
			//For some text piece, they're not just vertical or horizontal ones, and the rotation of them may be other value like 30, 70 and so on.
			//To simplify the testing, here, we only deal with vertical piece and horizontal pieces, which rotation should be 0, 90, 180, or 270.
			//For other rotation values, they will be treated as vertical or horizontal:
			//(315, 360] & [0, 45] --> as rotation = 0
			//(45, 135] --> as rotation = 90
			//(135, 225] --> as rotation = 180
			//(225, 315] --> as rotation = 270
			if ((rotation > 315 && rotation <=360)
				|| (rotation <= 45)
				|| (rotation > 135 && rotation <= 225)
				)
			{// As rotation = 0, or 180, to draw in the bottom of piece.
				quadPoints[i].x1 = rectOfPiece.left;
				quadPoints[i].y1 = rectOfPiece.top;
				quadPoints[i].x2 = rectOfPiece.right;
				quadPoints[i].y2 = rectOfPiece.top;
				quadPoints[i].x3 = rectOfPiece.left;
				quadPoints[i].y3 = rectOfPiece.bottom;
				quadPoints[i].x4 = rectOfPiece.right;
				quadPoints[i].y4 = rectOfPiece.bottom;
			}
			else if ((rotation > 45 && rotation <= 135)
				|| (rotation > 225 && rotation <= 315)
				)
			{// As rotation = 90, or 270, to draw in the left side of piece.
				quadPoints[i].x1 = rectOfPiece.right;
				quadPoints[i].y1 = rectOfPiece.top;
				quadPoints[i].x2 = rectOfPiece.right;
				quadPoints[i].y2 = rectOfPiece.bottom;
				quadPoints[i].x3 = rectOfPiece.left;
				quadPoints[i].y3 = rectOfPiece.top;
				quadPoints[i].x4 = rectOfPiece.left;
				quadPoints[i].y4 = rectOfPiece.bottom;
			}
		}
		//Set the quadrilaterals points of annotation.
		FSPDF_Annot_SetQuadPoints(annot, quadPoints, pieceCount);
		delete[] quadPoints;
	}
	FSPDF_TextPage_Release(hTextPage);
	FSPDF_TextSelection_Release(textSelection);

	//Set the stroke color and opacity of annotation.
	FSPDF_Annot_SetColor(annot, FALSE, 0x0000FF00);
	FSPDF_Annot_SetOpacity(annot, 1.0);
}

//Set FreeText annotation properties.
void CReader_Page::setFreeTextAnnotProperties(FSCRT_ANNOT annot, char* sAnnotType, FSCRT_RECTF rect)
{
	//Deal with the FreeText annotation only.
	if(strcmp(sAnnotType, FSPDF_ANNOTTYPE_FREETEXT)) return;

	//Set the alignment of annotation.
	FSPDF_Annot_SetAlignment(annot, 0);
	FSCRT_BSTR intent = {FSPDF_ANNOT_INTENTNAME_FREETEXT_CALLOUT, sizeof(FSPDF_ANNOT_INTENTNAME_FREETEXT_CALLOUT)-1};
	FSPDF_Annot_SetIntent(annot, &intent);

	FS_FLOAT width = rect.right - rect.left;
	FS_FLOAT height = rect.top - rect.bottom ;

	FSPDF_Annot_SetInnerRectMargin(annot, width/2, 0, 0, height/2);
	//FSPDF_Annot_SetRect(annot, &rect);

	FSCRT_BSTR startStyle;
	FSCRT_BSTR endStyle;
	FSCRT_BStr_Init(&startStyle);
	FSCRT_BStr_Init(&endStyle);
	FSCRT_BStr_Set(&startStyle, FSPDF_ANNOT_LINEENDINGSTYLE_SQUARE, strlen(FSPDF_ANNOT_LINEENDINGSTYLE_SQUARE));
	FSCRT_BStr_Set(&endStyle, "None", strlen("None"));

	FS_RESULT ret = FSPDF_Annot_SetLineEndingStyles(annot, &startStyle, &endStyle);
	FSCRT_BStr_Clear(&startStyle);
	FSCRT_BStr_Clear(&endStyle);

	FSPDF_Annot_SetCalloutLinePoints(annot, rect.left - 3, rect.bottom -3, rect.left + width/4, rect.top + height/4 , rect.left + width/2, rect.top + height/4);

	//Set the default appearance of annotation, which can be used in formatting the text.
	//Prepare the default appearance data.
	FSPDF_DEFAULTAPPEARANCE defAppearance;
	defAppearance.flags = 0;
	defAppearance.flags |= FSPDF_DEFAULTAPPEARANCE_FONT;
	FSCRT_BSTRC(simsun, "SimSun");
	ret = FSCRT_Font_Create(&simsun, 0, 400, FSCRT_CHARSET_GB2312, &defAppearance.font);
	defAppearance.fontSize = 12;

	defAppearance.flags |= FSPDF_DEFAULTAPPEARANCE_TEXTCOLOR;
	defAppearance.textColor = 0xff00ff00; //green

	//Set the default appearance of annotation.
	ret = FSPDF_Annot_SetDefaultAppearance(annot, &defAppearance);

	FSPDF_Annot_ResetAppearance(annot);

	FSCRT_Font_Release(defAppearance.font);
}

//Set Link annotation properties.
void CReader_Page::setLinkAnnotProperties(FSCRT_ANNOT annot, char* sAnnotType)
{
	//Deal with the Link annotation only.
	if(strcmp(sAnnotType, FSPDF_ANNOTTYPE_LINK)) return;

	//Set the highlighting mode of annotation.
	FSPDF_Annot_SetHighlightingMode(annot, FSPDF_ANNOT_HIGHLIGHTINGMODE_OUTLINE);

	//Insert the actions of specified index associated with the annotation.
	//Prepare the action data.
	FSPDF_ACTIONDATA data;
	data.actionType = FSPDF_ACTION_URI;
	//Prepare the URI action data.
	FSPDF_ACTIONDATA_URI setStrUrlNoAction;
	FSCRT_BStr_Init(&setStrUrlNoAction.url);
	FSCRT_BStr_Set(&setStrUrlNoAction.url, "www.google.com", (FS_DWORD)strlen("www.google.com"));
	setStrUrlNoAction.mapPosition = TRUE;
	data.actionData = (FS_LPVOID)&setStrUrlNoAction;

	//Insert the action data.
	FSPDF_Annot_InsertAction(annot, FSPDF_TRIGGER_ANNOT_MU, 0, &data);
	FSCRT_BStr_Clear(&setStrUrlNoAction.url);
}

void CReader_Page::setAnnotBorder(FSCRT_ANNOT annot)
{
	//Set the border of annotation.
	//Prepare FSPDF_ANNOTBORDER data.
	FSPDF_ANNOTBORDER border;
	border.borderWidth = 3.0;
	border.borderStyle = FSPDF_ANNOT_BORDERSTYLE_DASHED;
	border.cloudIntensity = -1;
	border.dashPhase = 5;
	border.dashCount = 2;
	border.dashPattern[0] = border.dashPattern[1] = 1.0;
	//Set the border.
	FSPDF_Annot_SetBorder(annot, &border);
}