/**
 * Copyright (C) 2001-2019, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a common source to implement some general functions which are used by examples.
 */

//Include common header file.
#include "fgsdk_common.h"

/*******************************************************************************/
/* Library management                                                          */
/*******************************************************************************/
//Extension to allocate memory buffer. Implementation to FSCRT_MEMMGRHANDLER::Alloc
static FS_LPVOID	FSDK_Alloc(FS_LPVOID clientData, FS_DWORD size)
{
	return malloc((size_t)size);
}

//Extension to reallocate memory buffer. Implementation to FSCRT_MEMMGRHANDLER::Realloc
static FS_LPVOID	FSDK_Realloc(FS_LPVOID clientData, FS_LPVOID ptr, FS_DWORD newSize)
{
	return realloc(ptr, (size_t)newSize);
}

//Extension to free memory buffer. Implementation to FSCRT_MEMMGRHANDLER::Free
static void			FSDK_Free(FS_LPVOID clientData, FS_LPVOID ptr)
{
	free(ptr);
}

//Global variables for extension manager
static FSCRT_MEMMGRHANDLER	g_MemMgrHandler = {NULL, FSDK_Alloc, FSDK_Realloc, FSDK_Free};
static FS_LPVOID			g_pGlobalBuffer = NULL;

//Default size of global buffer, 16MB
#define FSDK_GLOBALBUFFER_SIZE	(1024 * 1024 * 16)

GSDKExport FS_RESULT FSDK_CreateMgr(bool useDefMgr)
{
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;
	if (useDefMgr) //if use default manager
		ret = FSCRT_Library_CreateDefaultMgr();
	else //if use extension manager
	{
		g_pGlobalBuffer = malloc(FSDK_GLOBALBUFFER_SIZE);
		if (!g_pGlobalBuffer) return FSCRT_ERRCODE_ERROR;
		ret = FSCRT_Library_CreateMgr(g_pGlobalBuffer, FSDK_GLOBALBUFFER_SIZE, &g_MemMgrHandler);
	}

	return ret;
}

GSDKExport FS_RESULT FSDK_UnlockLibrary(const char* strLicenseID, const char* strUnlockCode)
{
	FSCRT_BSTR license_id;
	FSCRT_BStr_Init(&license_id);
	license_id.str = (FS_LPSTR)strLicenseID;
	license_id.len = strlen(strLicenseID);
	FSCRT_BSTR unlockCode;
	FSCRT_BStr_Init(&unlockCode);
	unlockCode.str = (FS_LPSTR)strUnlockCode;
	unlockCode.len = strlen(strUnlockCode);

	return FSCRT_License_UnlockLibrary(&license_id, &unlockCode);
}

GSDKExport void FSDK_FinalizeLibrary()
{
	//Destroy manager
	FSCRT_Library_DestroyMgr();

	//Free global buffer for extension manager
	if (g_pGlobalBuffer)
	{
		free(g_pGlobalBuffer);
		g_pGlobalBuffer = NULL;
	}
}

GSDKExport FS_RESULT FSDK_LicenseGetType(FS_INT32* type)
{
	return FSCRT_License_GetType(type);
}

GSDKExport FS_RESULT FSDK_InitPDFModule()
{
	return FSCRT_PDFModule_Initialize();
}

GSDKExport FS_RESULT FSDK_FinalizePDFModule()
{
	return FSCRT_PDFModule_Finalize();
}

//Structure for file data
typedef struct _FSDK_FILEDATA
{
	FILE*				fileStream;		//file stream object
	FSCRT_FILEHANDLER	fileHandler;	//file handler of Foxit SDK
} FSDK_FILEDATA;

//Implementation to FSCRT_FILEHANDLER::Release
static void FSDK_FileHandler_Release(FS_LPVOID clientData)
{
	//Check parameter
	if (!clientData) return;
	//Close file stream
	fclose(((FSDK_FILEDATA*)clientData)->fileStream);
	//Free file data
	free((FSDK_FILEDATA*)clientData);
}

//Implementation to FSCRT_FILEHANDLER::GetSize
static FS_DWORD	FSDK_FileHandler_GetSize(FS_LPVOID clientData)
{
	//Check parameter
	if (!clientData) return 0;

	//Move file pointer to the end, and get file size
	long curPos = ftell(((FSDK_FILEDATA*)clientData)->fileStream);
	fseek(((FSDK_FILEDATA*)clientData)->fileStream, 0, SEEK_END);
	long size = ftell(((FSDK_FILEDATA*)clientData)->fileStream);
	fseek(((FSDK_FILEDATA*)clientData)->fileStream, curPos, SEEK_SET);

	return (FS_DWORD)size;
}

//Implementation to FSCRT_FILEHANDLER::ReadBlock
static FS_RESULT FSDK_FileHandler_ReadBlock(FS_LPVOID clientData, FS_DWORD offset, FS_LPVOID buffer, FS_DWORD size)
{
	//Check parameter
	if (!clientData) return FSCRT_ERRCODE_FILE;

	//Change file pointer to offset
	fseek(((FSDK_FILEDATA*)clientData)->fileStream, (long)offset, SEEK_SET);
	//Read data
	size_t readSize = fread(buffer, 1, size, ((FSDK_FILEDATA*)clientData)->fileStream);
	return readSize == size ? FSCRT_ERRCODE_SUCCESS : FSCRT_ERRCODE_FILE;
}

//Implementation to FSCRT_FILEHANDLER::WriteBlock
static FS_RESULT FSDK_FileHandler_WriteBlock(FS_LPVOID clientData, FS_DWORD offset, FS_LPCVOID buffer, FS_DWORD size)
{
	//Check parameter
	if (!clientData) return FSCRT_ERRCODE_FILE;

	//Change file pointer to offset
	fseek(((FSDK_FILEDATA*)clientData)->fileStream, (long)offset, SEEK_SET);
	//Write data
	size_t writeSize = fwrite(buffer, 1, size, ((FSDK_FILEDATA*)clientData)->fileStream);
	return writeSize == size ? FSCRT_ERRCODE_SUCCESS : FSCRT_ERRCODE_FILE;
}

//Implementation to FSCRT_FILEHANDLER::Flush
static FS_RESULT FSDK_FileHandler_Flush(FS_LPVOID clientData)
{
	//Check parameter
	if (!clientData) return FSCRT_ERRCODE_FILE;

	//Flush file
	fflush(((FSDK_FILEDATA*)clientData)->fileStream);

	return FSCRT_ERRCODE_SUCCESS;
}

//Implementation to FSCRT_FILEHANDLER::Truncate
static FS_RESULT FSDK_FileHandler_Truncate(FS_LPVOID clientData, FS_DWORD size)
{
	//Unsupported
	return FSCRT_ERRCODE_UNSUPPORTED;
}

GSDKExport FSCRT_FILE FSDK_OpenFileW(const wchar_t* fileName, const wchar_t* fileMode, bool useDef)
{
	//Check parameters, return NULL if they are invalid
	if (!fileName || wcslen(fileName) < 1 || !fileMode) return NULL;

	if (useDef) //if use default implementation
	{
		//Check file modes
		FS_DWORD fileModes = 0;
		std::wstring fm(fileMode);
		if (fm.find('+') > -1)
			fileModes = FSCRT_FILEMODE_WRITE;
		else if (fm.find('w') > -1)
			fileModes = FSCRT_FILEMODE_TRUNCATE;
		else
			fileModes = FSCRT_FILEMODE_READONLY;

		char* old_locale =  setlocale(LC_CTYPE,NULL);
		setlocale( LC_ALL, "" );

		size_t length = wcslen(fileName);
		wchar_t* buf = new wchar_t[length*2];
		memset(buf, 0 ,sizeof(wchar_t)*length);

		setlocale( LC_CTYPE, old_locale);

		FSCRT_BSTR bstrFile;
		FSCRT_BStr_Init(&bstrFile);
		if(sizeof(wchar_t)==2)
			FSCRT_UTF8_FromUTF16LE(&bstrFile, (FS_WORD*)buf, length);  //windwos use
		else
			FSCRT_UTF8_FromUTF32LE(&bstrFile, (FS_DWORD*)buf, length);  //linux use

		delete[] buf;

		FSCRT_FILE file = NULL;
		//Call FSCRT_File_CreateFromFileName to open file
		FS_RESULT ret = FSCRT_File_CreateFromFileName(&bstrFile, fileModes, &file);
		//Clear string object
		FSCRT_BStr_Clear(&bstrFile);
		return file;
	}

	//For extension implementation

	//Open file as stream
	FILE* fileStream = _wfopen(fileName, fileMode);
	if (!fileStream) return NULL;
	//Allocate file data
	FSDK_FILEDATA* fileData = (FSDK_FILEDATA*)malloc(sizeof(FSDK_FILEDATA));
	if (!fileData)
	{
		//If error, close file stream and exit
		fclose(fileStream);
		return NULL;
	}
	//Initialize file handler
	fileData->fileHandler.clientData = fileData;
	fileData->fileHandler.Release = FSDK_FileHandler_Release;
	fileData->fileHandler.GetSize = FSDK_FileHandler_GetSize;
	fileData->fileHandler.ReadBlock = FSDK_FileHandler_ReadBlock;
	fileData->fileHandler.WriteBlock = FSDK_FileHandler_WriteBlock;
	fileData->fileHandler.Flush = FSDK_FileHandler_Flush;
	fileData->fileHandler.Truncate = FSDK_FileHandler_Truncate;
	fileData->fileStream = fileStream;
	//Call FSCRT_File_Create to create Foxit GSDK file object
	FSCRT_FILE file = NULL;
	if (FSCRT_File_Create(&fileData->fileHandler, &file) == FSCRT_ERRCODE_SUCCESS)
	{
		//Success, return file object
		return file;
	}

	//Free file data, close file stream and exit
	free(fileData);
	fclose(fileStream);
	return NULL;
}

//Structure for file data
typedef struct _FSDK_FILEDATA_WIN
{
	HANDLE				hFile; 		    //file handle
	FSCRT_FILEHANDLER	fileHandler;	//file handler of Foxit SDK
} FSDK_FILEDATA_WIN;

//Implementation to FSCRT_FILEHANDLER::Release
static void FSDK_FileHandler_Release_Win(FS_LPVOID clientData)
{
	//Check parameter
	if (!clientData) return;
	//Close file handle
	CloseHandle(((FSDK_FILEDATA_WIN*)clientData)->hFile);
	//Free file data
	free((FSDK_FILEDATA_WIN*)clientData);
}

//Implementation to FSCRT_FILEHANDLER::GetSize
static FS_DWORD	FSDK_FileHandler_GetSize_Win(FS_LPVOID clientData)
{
	//Check parameter
	if (!clientData) return 0;

	//Move file pointer to the end, and get file size
	FS_DWORD size = GetFileSize(((FSDK_FILEDATA_WIN*)clientData)->hFile, NULL);

	return size;
}

//Implementation to FSCRT_FILEHANDLER::ReadBlock
static FS_RESULT FSDK_FileHandler_ReadBlock_Win(FS_LPVOID clientData, FS_DWORD offset, FS_LPVOID buffer, FS_DWORD size)
{
	//Check parameter
	if (!clientData) return FSCRT_ERRCODE_FILE;
	//Change file pointer to offset
	SetFilePointer(((FSDK_FILEDATA_WIN*)clientData)->hFile, offset, NULL, FILE_BEGIN);

	//Read data
	size_t readSize = 0;
	ReadFile(((FSDK_FILEDATA_WIN*)clientData)->hFile, buffer, size, (LPDWORD)&readSize, NULL);
	return readSize == size ? FSCRT_ERRCODE_SUCCESS : FSCRT_ERRCODE_FILE;
}

//Implementation to FSCRT_FILEHANDLER::WriteBlock
static FS_RESULT FSDK_FileHandler_WriteBlock_Win(FS_LPVOID clientData, FS_DWORD offset, FS_LPCVOID buffer, FS_DWORD size)
{
	//Check parameter
	if (!clientData) return FSCRT_ERRCODE_FILE;

	//Change file pointer to offset
	SetFilePointer(((FSDK_FILEDATA_WIN*)clientData)->hFile, offset, NULL, FILE_BEGIN);
	//Write data
	size_t writeSize = 0;
	WriteFile(((FSDK_FILEDATA_WIN*)clientData)->hFile, buffer, size, (LPDWORD)&writeSize, NULL);
	return writeSize == size ? FSCRT_ERRCODE_SUCCESS : FSCRT_ERRCODE_FILE;
}

//Implementation to FSCRT_FILEHANDLER::Flush
static FS_RESULT FSDK_FileHandler_Flush_Win(FS_LPVOID clientData)
{
	//Check parameter
	if (!clientData) return FSCRT_ERRCODE_FILE;

	//Flush file
	FlushFileBuffers(((FSDK_FILEDATA_WIN*)clientData)->hFile);

	return FSCRT_ERRCODE_SUCCESS;
}

//Implementation to FSCRT_FILEHANDLER::Truncate
static FS_RESULT FSDK_FileHandler_Truncate_Win(FS_LPVOID clientData, FS_DWORD size)
{
	//Unsupported
	return FSCRT_ERRCODE_UNSUPPORTED;
}

GSDKExport FSCRT_FILE FSDK_OpenFileW_Win(const TCHAR* fileName)
{
	//Check parameters, return NULL if they are invalid
	if (!fileName || wcslen(fileName) < 1 ) return NULL;

	//For extension implementation

	//Open file as stream
	HANDLE hFile = NULL;
	hFile = ::CreateFile(fileName, GENERIC_READ,  FILE_SHARE_READ|FILE_SHARE_WRITE, NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);      
	if (hFile == INVALID_HANDLE_VALUE) return NULL;
	//Allocate file data
	FSDK_FILEDATA_WIN* fileData = (FSDK_FILEDATA_WIN*)malloc(sizeof(FSDK_FILEDATA_WIN));
	if (!fileData)
	{
		//If error, close file stream and exit
		CloseHandle(hFile);
		return NULL;
	}
	//Initialize file handler
	fileData->fileHandler.clientData = fileData;
	fileData->fileHandler.Release = FSDK_FileHandler_Release_Win;
	fileData->fileHandler.GetSize = FSDK_FileHandler_GetSize_Win;
	fileData->fileHandler.ReadBlock = FSDK_FileHandler_ReadBlock_Win;
	fileData->fileHandler.WriteBlock = FSDK_FileHandler_WriteBlock_Win;
	fileData->fileHandler.Flush = FSDK_FileHandler_Flush_Win;
	fileData->fileHandler.Truncate = FSDK_FileHandler_Truncate_Win;
	fileData->hFile = hFile;
	//Call FSCRT_File_Create to create Foxit GSDK file object
	FSCRT_FILE file = NULL;
	if (FSCRT_File_Create(&fileData->fileHandler, &file) == FSCRT_ERRCODE_SUCCESS)
	{
		//Success, return file object
		return file;
	}

	//Free file data, close file stream and exit
	free(fileData);
	CloseHandle(hFile);
	return NULL;
}

GSDKExport FS_RESULT FSDK_CloseFile(FSCRT_FILE file)
{
	return FSCRT_File_Release(file);
}

//app handler callback
FS_RESULT Form_OnRecover(FS_LPVOID clientData, FS_LPVOID senderObject, FS_DWORD senderObjectType)
{
	return FSCRT_ERRCODE_SUCCESS;
}

GSDKExport FS_RESULT FSDK_SetAppHander()
{
	FSCRT_APPHANDLER appHandler;
	memset(&appHandler, 0, sizeof(FSCRT_APPHANDLER));
	appHandler.OnRecover = Form_OnRecover;
	return FSCRT_Library_SetAppHandler(&appHandler);
}

GSDKExport FS_RESULT FSDK_ProgressContinue(FSCRT_PROGRESS progress)
{
	return FSCRT_Progress_Continue(progress, NULL);
}

GSDKExport FS_RESULT FSDK_CreateBitmap(FS_INT32 width, FS_INT32 height, FS_INT32 format, FS_LPVOID buffer, FS_INT32 stride, FSCRT_BITMAP* bitmap)
{
	return FSCRT_Bitmap_Create(width, height, format, buffer, stride, bitmap);
}

GSDKExport FS_RESULT FSDK_ReleaseBitmap(FSCRT_BITMAP bitmap,  FS_LPVOID* buffer)
{
	int ret = FSCRT_Bitmap_Release(bitmap);
	if (buffer)
		delete[] buffer;
	return ret;
}

GSDKExport int FSDK_GetBitmapSize(FSCRT_BITMAP bitmap)
{
	FSCRT_BITMAPINFO* bitmapInfo = NULL;
	FS_DWORD psize = 0;
	FSCRT_Bitmap_GetBitmapInfo(bitmap, bitmapInfo, &psize);
	bitmapInfo = (FSCRT_BITMAPINFO*)malloc(psize);
	FSCRT_Bitmap_GetBitmapInfo(bitmap, bitmapInfo, &psize);
	int size = 14 + 40 + bitmapInfo->bmiHeader.sizeImage;
	
    free(bitmapInfo);
	return size;
}

GSDKExport FS_RESULT FSDK_GetBitmapBuffer(FSCRT_BITMAP bitmap, FS_LPVOID* buffer)
{
	// Calculate file size
	FSCRT_BITMAPINFO* bitmapInfo = NULL;
	FS_DWORD psize = 0;
	FSCRT_Bitmap_GetBitmapInfo(bitmap, bitmapInfo, &psize);
	bitmapInfo = (FSCRT_BITMAPINFO*)malloc(psize);
	FSCRT_Bitmap_GetBitmapInfo(bitmap, bitmapInfo, &psize);
	int size = 14 + 40 + bitmapInfo->bmiHeader.sizeImage;
	int bufUsedLength = 0;

	unsigned char* data = new unsigned char[size];
	// write file header
	data[0] = 'B';
	data[1] = 'M';
	data[2] = (unsigned char)size;
	data[3] = (unsigned char)(size >> 8);
	data[4] = (unsigned char)(size >> 16);
	data[5] = (unsigned char)(size >> 24);
	data[6] = data[7] = data[8] = data[9] = 0;
	data[10] = 54;
	data[11] = data[12] = data[13] = 0;
	bufUsedLength = 14;
	// write bitmap header
	int width = bitmapInfo->bmiHeader.width;
	int height = bitmapInfo->bmiHeader.height;

	memset(data + bufUsedLength, 0, 40);
	data[bufUsedLength+0] = 40;
	data[bufUsedLength+4] = (unsigned char)width;
	data[bufUsedLength+5] = (unsigned char)(width >> 8);
	data[bufUsedLength+6] = (unsigned char)(width >> 16);
	data[bufUsedLength+7] = (unsigned char)(width >> 24);
	data[bufUsedLength+8] = (unsigned char)(height);
	data[bufUsedLength+9] = (unsigned char)((height) >> 8);
	data[bufUsedLength+10] = (unsigned char)((height) >> 16);
	data[bufUsedLength+11] = (unsigned char)((height) >> 24);
	data[bufUsedLength+12] = 1;
	data[bufUsedLength+14] = (char)bitmapInfo->bmiHeader.bitCount;

	bufUsedLength += 40;

	// Write pixel data
	int stribe;
	FSCRT_Bitmap_GetLineStride(bitmap, &stribe);
	for (int row = 0; row < (-height); row ++) {
		void* buf = NULL;
		FSCRT_Bitmap_GetLineBuffer(bitmap, row, &buf);
		memcpy(data + bufUsedLength, buf, stribe);
		bufUsedLength += stribe;
	}

	free(bitmapInfo);
	*buffer = (void*)data;
	return 0;
}

GSDKExport FS_RESULT FSDK_FillBitmapRect(FSCRT_BITMAP bitmap, FS_ARGB color, int left, int top, int right, int bottom)
{
	FSCRT_RECT fillRect;
	fillRect.left = left;
	fillRect.top = top;
	fillRect.right = right;
	fillRect.bottom = bottom;

	return FSCRT_Bitmap_FillRect(bitmap, color, &fillRect);
}

GSDKExport FS_RESULT FSDK_RenderCreateOnBitmap(FSCRT_BITMAP bitmap, FSCRT_RENDERER* renderer)
{
	return FSCRT_Renderer_CreateOnBitmap(bitmap, renderer);
}

GSDKExport FS_RESULT FSDK_RenderPageByGE(FSCRT_PAGE page, int width, int height, int rotate, FSCRT_BITMAP bitmap)
{
	FSCRT_RENDERER renderer;
	FS_RESULT ret = FSCRT_Renderer_CreateOnBitmap(bitmap, &renderer);
	if (ret != FSCRT_ERRCODE_SUCCESS) return ret;

	FSCRT_MATRIX matrix;
	FSPDF_Page_GetMatrix(page, 0, 0, width, height, rotate, &matrix);

	FSPDF_RENDERCONTEXT context;
	ret = FSPDF_RenderContext_Create(&context);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSCRT_Renderer_Release(renderer);
		return ret;
	}

	FSPDF_RenderContext_SetMatrix(context, &matrix);

	FSPDF_RenderContext_SetFlags(context, FSPDF_RENDERCONTEXTFLAG_ANNOT|FSPDF_RENDERCONTEXTFLAG_NOWIDGET);

	FSCRT_PROGRESS progress;
	ret = FSPDF_RenderContext_StartPage(context, renderer, page, FSPDF_PAGERENDERFLAG_NORMAL, &progress);
	if (ret != FSCRT_ERRCODE_SUCCESS) 
	{
		FSPDF_RenderContext_Release(context);
		FSCRT_Renderer_Release(renderer);
		return ret;
	}

	ret = FSCRT_ERRCODE_TOBECONTINUED;
	while (ret == FSCRT_ERRCODE_TOBECONTINUED)
	{
		ret = FSCRT_Progress_Continue(progress, NULL);
	}
	FSCRT_Progress_Release(progress);

	progress = NULL;
	ret = FSPDF_RenderContext_StartPageFormControls(context, renderer, page, &progress);
	ret = FSCRT_ERRCODE_TOBECONTINUED;
	while (ret == FSCRT_ERRCODE_TOBECONTINUED)
	{
		ret = FSCRT_Progress_Continue(progress, NULL);
	}
	FSCRT_Progress_Release(progress);

	FSPDF_RenderContext_Release(context);
	FSCRT_Renderer_Release(renderer);	

	if (ret == FSCRT_ERRCODE_SUCCESS || ret == FSCRT_ERRCODE_FINISHED)
		return FSCRT_ERRCODE_SUCCESS;
	else
		return ret;
}